@@ -1,4 +1,5 @@
-#!/bin/bash -e
+#!/usr/bin/env bash
+set -e
 args=()
 
 while [[ $# > 0 ]];
