@@ -1,9 +1,9 @@
-;;; cg.el --- major mode for editing Constraint Grammar files
+;;; cg.el --- major mode for editing Constraint Grammar files  -*- lexical-binding: t; coding: utf-8 -*-
 
-;; Copyright (C) 2010-2013 Kevin Brubeck Unhammer
+;; Copyright (C) 2010-2017 Kevin Brubeck Unhammer
 
 ;; Author: Kevin Brubeck Unhammer <unhammer@fsfe.org>
-;; Version: 0.1.7
+;; Version: 0.3.0
 ;; Url: http://beta.visl.sdu.dk/constraint_grammar.html
 ;; Keywords: languages
 
@@ -24,6 +24,13 @@
 
 ;;; Commentary:
 
+;; This package provides a major mode for editing Constraint Grammar
+;; source files, including syntax highlighting and interactive grammar
+;; development from within Emacs.  Use `C-c C-i' to edit the text you
+;; want to edit, then just `C-c C-c' whenever you want to run the
+;; grammar over that text.  Clicking on a line number in the trace
+;; output will take you to the definition of that rule.
+
 ;; Usage:
 ;;
 ;; (autoload 'cg-mode "/path/to/cg.el"
@@ -32,17 +39,19 @@
 ;; ; Or if you use a non-standard file suffix, e.g. .rlx:
 ;; (add-to-list 'auto-mode-alist '("\\.rlx\\'" . cg-mode))
 
-;; I recommend using auto-complete-mode for tab-completion, and
-;; smartparens-mode if you're used to it (paredit-mode does not work
-;; well if you have set names with the # character in them). Both are
-;; available from MELPA (see http://melpa.milkbox.net/). You can
-;; lazy-load auto-complete for cg-mode like this:
+;; I recommend using `company-mode' for tab-completion, and
+;; `smartparens-mode' if you're used to it (`paredit-mode' does not
+;; work well if you have set names with the # character in them). Both
+;; are available from MELPA (see http://melpa.milkbox.net/).
 ;;
-;; (eval-after-load 'auto-complete '(add-to-list 'ac-modes 'cg-mode))
+;; You can lazy-load company-mode for cg-mode like this:
+;;
+;; (eval-after-load 'company-autoloads
+;;     '(add-hook 'cg-mode-hook #'company-mode))
 
 
 ;; TODO:
-;; - optionally highlight any LIST/SET without ; at the end
+;; - investigate bug in `show-smartparens-mode' causing slowness
 ;; - different syntax highlighting for sets and tags (difficult)
 ;; - use something like prolog-clause-start to define M-a/e etc.
 ;; - run vislcg3 --show-unused-sets and buttonise with line numbers (like Occur does)
@@ -50,20 +59,17 @@
 ;; - the rest of the keywords
 ;; - http://beta.visl.sdu.dk/cg3/single/#regex-icase
 ;; - keyword tab-completion
-;; - the quotes-within-quotes thing plays merry hell with
-;;   paredit-doublequote, write a new doublequote function?
-;; - font-lock-syntactic-keywords is obsolete since 24.1
-;; - derive cg-mode from prog-mode?
+;; - `font-lock-syntactic-keywords' is obsolete since 24.1
 ;; - goto-set/list
 ;; - show definition of set/list-at-point in modeline
-;; - send dictionary to auto-complete
 ;; - show section name/number in modeline
 
 ;;; Code:
 
-(defconst cg-version "0.1.7" "Version of cg-mode")
+(defconst cg-version "0.3.0" "Version of cg-mode.")
 
 (eval-when-compile (require 'cl))
+(require 'cl-lib)
 
 ;;;============================================================================
 ;;;
@@ -103,11 +109,10 @@ See also `cg-command'."
 
 ;;;###autoload
 (defcustom cg-pre-pipe "cg-conv"
-  "Pipeline to run before the vislcg3 command when testing a file
-with `cg-check'.
+  "Pipeline to run before vislcg3 when testing a file with `cg-check'.
 
 Buffer-local, so use `setq-default' if you want to change the
-global default value. If you want to set it on a per-file basis,
+global default value.  If you want to set it on a per-file basis,
 put a line like
 
 # -*- cg-pre-pipe: \"lt-proc foo.bin | cg-conv\"; othervar: value; -*-
@@ -120,11 +125,10 @@ See also `cg-command' and `cg-post-pipe'."
 
 ;;;###autoload
 (defcustom cg-post-pipe ""
-  "Pipeline to run after the vislcg3 command when testing a file
-with `cg-check'.
+  "Pipeline to run after vislcg3 when testing a file with `cg-check'.
 
 Buffer-local, so use `setq-default' if you want to change the
-global default value. If you want to set it on a per-file basis,
+global default value.  If you want to set it on a per-file basis,
 put a line like
 
 # -*- cg-post-pipe: \"cg-conv --out-apertium | lt-proc -b foo.bin\"; -*-
@@ -140,9 +144,10 @@ See also `cg-command' and `cg-pre-pipe'."
     ;; These are not sets (and don't have names after the kw) but we
     ;; have them here to make beginning-of-defun work:
     "MAPPING-PREFIX" "SOFT-DELIMITERS" "DELIMITERS")
-  "Used for indentation, highlighting etc.; don't change without
-re-evaluating `cg-kw-re' (or all of cg.el).")
-(defconst cg-kw-set-re (regexp-opt cg-kw-set-list))
+  "List-like keywords used for indentation, highlighting etc.
+Don't change without re-evaluating `cg-kw-re' (or all of cg.el).")
+(defconst cg-kw-set-re (regexp-opt cg-kw-set-list)
+  "Regexp version of `cg-kw-set-list'.")
 
 (defconst cg-kw-rule-list
   '("SUBSTITUTE"
@@ -159,10 +164,12 @@ re-evaluating `cg-kw-re' (or all of cg.el).")
     "ADDRELATIONS" "REMRELATIONS" "SETRELATIONS"
     "SETVARIABLE"  "REMVARIABLE"
     "APPEND")
-  "Used for indentation, highlighting etc.; don't change without
-re-evaluating `cg-kw-re' (or all of cg.el)." )
-(defconst cg-kw-rule-re (regexp-opt cg-kw-rule-list))
-(defconst cg-kw-re (regexp-opt (append cg-kw-set-list cg-kw-rule-list)))
+  "Rule-starter keywords for indentation, highlighting etc.
+Don't change without re-evaluating `cg-kw-re' (or all of cg.el)." )
+(defconst cg-kw-rule-re (regexp-opt cg-kw-rule-list)
+    "Regexp version of `cg-kw-rule-list'.")
+(defconst cg-kw-re (regexp-opt (append cg-kw-set-list cg-kw-rule-list))
+  "Regexp combination of `cg-kw-rule-list' and `cg-kw-set-list'.")
 
 (defconst cg-kw-rule-flags '("NEAREST"
 			     "ALLOWLOOP"
@@ -189,9 +196,9 @@ re-evaluating `cg-kw-re' (or all of cg.el)." )
 			     "REVERSE"
 			     "SUB"
 			     "OUTPUT")
-  "Used for highlighting, from
-  http://visl.sdu.dk/svn/visl/tools/vislcg3/trunk/src/Strings.cpp
-  Don't change without re-evaluating the file.")
+  "Rule flags used for highlighting.
+from http://visl.sdu.dk/svn/visl/tools/vislcg3/trunk/src/Strings.cpp
+Don't change without re-evaluating the file.")
 (defconst cg-kw-context-flags '("NOT"
 				"NEGATE"
 				"NONE"
@@ -205,8 +212,8 @@ re-evaluating `cg-kw-re' (or all of cg.el)." )
 				"BEFORE"
 				"WITH"
 				"TO")
-  "Used for highlighting; Don't change without re-evaluating the
-  file.")
+  "Context flags used for highlighting.
+Don't change without re-evaluating the file.")
 (defconst cg-kw-flags-re (regexp-opt (append cg-kw-rule-flags cg-kw-context-flags)))
 
 
@@ -256,18 +263,13 @@ re-evaluating `cg-kw-re' (or all of cg.el)." )
     ;; using syntactic keywords for "
     (modify-syntax-entry ?\" "." table)
     (modify-syntax-entry ?» "." table)
-  (modify-syntax-entry ?« "." table)
-                       table))
-
-(defun cg-syntax-at-pos ()
-  (let ((ppss (syntax-ppss)))
-    (cond
-     ((nth 8 ppss) (if (nth 4 ppss) 'comment 'string))
-     ((nth 1 ppss) 'paren))))
+    (modify-syntax-entry ?« "." table)
+    table)
+  "Syntax table for CG mode.")
 
 (defun cg-beginning-of-defun ()
   (re-search-backward defun-prompt-regexp nil 'noerror)
-  (while (cg-syntax-at-pos)
+  (while (nth 4 (syntax-ppss))
     (re-search-backward defun-prompt-regexp nil 'noerror))
   (re-search-backward "\"<[^\"]>\"" (line-beginning-position) 'noerror))
 
@@ -275,55 +277,124 @@ re-evaluating `cg-kw-re' (or all of cg.el)." )
   (and (search-forward ";")
        (re-search-forward defun-prompt-regexp nil 'noerror)
        (goto-char (match-beginning 0)))
-  (while (cg-syntax-at-pos)
+  (while (nth 4 (syntax-ppss))
     (and (search-forward ";")
-	 (re-search-forward defun-prompt-regexp nil 'noerror)
-	 (goto-char (match-beginning 0))))
+         (re-search-forward defun-prompt-regexp nil 'noerror)
+         (goto-char (match-beginning 0))))
   (re-search-backward "\"<[^\"]>\"" (line-beginning-position) 'noerror))
 
+(defun cg--line-commented-p ()
+  (save-excursion
+    (back-to-indentation)
+    (looking-at "#")))
+
+(defun cg--region-commented-p (beg end)
+  (catch 'ret
+    (save-excursion
+      (goto-char beg)
+      (while (and (< (point) end)
+                  (< (point) (point-max)))
+        (if (cg--line-commented-p)
+            (forward-line)
+          (throw 'ret nil)))
+      (throw 'ret t))))
+
+(defun cg--comment/uncomment-rule (comment &optional n)
+  "Comment/uncomment a rule around point."
+  (let ((i 0)
+        (n (if (numberp n) n 1))
+        (initial-point (point-marker)))
+    (while (< i n)
+      (incf i)
+      (let* ((r (save-excursion
+                  (if (search-forward ";" nil 'noerror)
+                      (1+ (point-marker))
+                    (point-max))))
+             (l (save-excursion
+                  (goto-char r)
+                  (if (re-search-backward defun-prompt-regexp nil 'noerror)
+                      (goto-char (line-beginning-position))
+                    (point-min)))))
+        ;; Only uncomment rules if they're completely commented (but
+        ;; always uncomment the first one)
+        (when (or comment
+                  (= i 1)
+                  (cg--region-commented-p l r))
+          (goto-char r)
+          (skip-chars-forward "\r\n[:blank:]")
+          (if comment
+              (comment-region l r)
+            (uncomment-region l r))
+          (skip-chars-forward "\r\n[:blank:]")))
+      (when (= n 1)
+        (goto-char initial-point)))))
+
+(defun cg-comment-rule (&optional n)
+  "Comment a rule around point.
+With a prefix argument N, comment that many rules."
+  (interactive "p")
+  (cg--comment/uncomment-rule 'comment n))
+
+(defun cg-uncomment-rule (&optional n)
+  "Uncomment a rule around point.
+With a prefix argument N, uncomment that many rules."
+  (interactive "p")
+  (cg--comment/uncomment-rule nil n))
+
+(defun cg-comment-or-uncomment-rule (&optional n)
+  "Comment the rule at point.
+If already inside (or before) a comment, uncomment instead.
+With a prefix argument N, (un)comment that many rules."
+  (interactive "p")
+  (if (or (elt (syntax-ppss) 4)
+          (< (save-excursion
+               (skip-chars-forward "\r\n[:blank:]")
+               (point))
+             (save-excursion
+               (comment-forward 1)
+               (point))))
+      (cg-uncomment-rule n)
+    (cg-comment-rule n)))
+
+
 ;;;###autoload
-(defun cg-mode ()
+(define-derived-mode cg-mode prog-mode "CG"
   "Major mode for editing Constraint Grammar files.
 
 CG-mode provides the following specific keyboard key bindings:
 
 \\{cg-mode-map}"
-  (interactive)
-  (kill-all-local-variables)
-  (setq major-mode 'cg-mode
-        mode-name "CG")
-  (use-local-map cg-mode-map)
-  (make-local-variable 'comment-start)
-  (make-local-variable 'comment-start-skip)
-  (make-local-variable 'font-lock-defaults)
-  (make-local-variable 'indent-line-function)
-  (setq comment-start "#"
-        comment-start-skip "#+[\t ]*"
-        font-lock-defaults
-        `((cg-font-lock-keywords cg-font-lock-keywords-1 cg-font-lock-keywords-2)
-          nil				; KEYWORDS-ONLY
-          'case-fold ; some keywords (e.g. x vs X) are case-sensitive,
+  :group 'cg
+  ;; Font lock
+  (set (make-local-variable 'font-lock-defaults)
+       `((cg-font-lock-keywords cg-font-lock-keywords-1 cg-font-lock-keywords-2)
+         nil				; KEYWORDS-ONLY
+         'case-fold ; some keywords (e.g. x vs X) are case-sensitive,
                                         ; but that doesn't matter for highlighting
-          ((?/ . "w") (?~ . "w") (?. . "w") (?- . "w") (?_ . "w"))
-          nil ;	  beginning-of-line		; SYNTAX-BEGIN
-          (font-lock-syntactic-keywords . cg-font-lock-syntactic-keywords)
-          (font-lock-syntactic-face-function . cg-font-lock-syntactic-face-function)))
-  (make-local-variable 'cg-mode-syntax-table)
-  (set-syntax-table cg-mode-syntax-table)
+         ((?/ . "w") (?~ . "w") (?. . "w") (?- . "w") (?_ . "w"))
+         nil ;	  beginning-of-line		; SYNTAX-BEGIN
+         (font-lock-syntactic-keywords . cg-font-lock-syntactic-keywords)
+         (font-lock-syntactic-face-function . cg-font-lock-syntactic-face-function)))
+  ;; Indentation
+  (set (make-local-variable 'indent-line-function) #'cg-indent-line)
+  ;; Comments and blocks
+  (set (make-local-variable 'comment-start) "#")
+  (set (make-local-variable 'comment-start-skip) "#+[\t ]*")
+  (set (make-local-variable 'comment-use-syntax) t)
   (set (make-local-variable 'parse-sexp-ignore-comments) t)
   (set (make-local-variable 'parse-sexp-lookup-properties) t)
-  (set (make-local-variable 'defun-prompt-regexp) cg-kw-re)
+  (set (make-local-variable 'defun-prompt-regexp) (concat cg-kw-re "\\(?::[^\n\t ]+\\)[\t ]"))
   (set (make-local-variable 'beginning-of-defun-function) #'cg-beginning-of-defun)
   (set (make-local-variable 'end-of-defun-function) #'cg-end-of-defun)
-  (setq indent-line-function #'cg-indent-line)
+
   (when font-lock-mode
     (setq font-lock-set-defaults nil)
     (font-lock-set-defaults)
+    ;; TODO: emacs 25 prefers `font-lock-ensure' and `font-lock-flush' over fontify
     (font-lock-fontify-buffer))
   (add-hook 'after-change-functions #'cg-after-change nil 'buffer-local)
   (let ((buf (current-buffer)))
-    (run-with-idle-timer 1 'repeat 'cg-output-hl buf))
-  (run-mode-hooks 'cg-mode-hook))
+    (run-with-idle-timer 1 'repeat 'cg-output-hl buf)))
 
 
 (defconst cg-font-lock-syntactic-keywords
@@ -344,13 +415,15 @@ CG-mode provides the following specific keyboard key bindings:
     ("[( \t\n]\\(\\^\\)" 1 "'")))
 
 (defun cg-font-lock-syntactic-face-function (state)
-  "Determine which face to use when fontifying syntactically. See
-`font-lock-syntactic-face-function'.
+  "Determine which face to use when fontifying syntactically.
 
-TODO: something like
-	((= 0 (nth 0 state)) font-lock-variable-name-face)
-would be great to differentiate SETs from their members, but it
-seems this function only runs on comments and strings..."
+Argument STATE is assumed to be from `parse-partial-sexp' at the
+beginning of the region to highlight; see
+`font-lock-syntactic-face-function'."
+  ;; TODO: something like
+  ;; 	((= 0 (nth 0 state)) font-lock-variable-name-face)
+  ;; would be great to differentiate SETs from their members, but it
+  ;; seems this function only runs on comments and strings...
   (cond ((nth 3 state)
          (if
              (save-excursion
@@ -390,23 +463,25 @@ seems this function only runs on comments and strings..."
   (let ((origin (point))
         (old-case-fold-search case-fold-search))
     (setq case-fold-search nil)		; for re-search-backward
-    (save-excursion
-      (let ((kw-pos (progn
-                      (goto-char (1- (or (search-forward ";" (line-end-position) t)
-                                         (line-end-position))))
-                      (re-search-backward cg-kw-re nil 'noerror))))
-        (setq case-fold-search old-case-fold-search)
-        (when kw-pos
-          (let* ((kw (match-string-no-properties 0)))
-            (if (and (not (equal kw ";"))
-                     (> origin (line-end-position)))
-                cg-indentation
-              0)))))))
+    (prog1
+        (save-excursion
+          (let ((kw-pos (progn
+                          (goto-char (1- (or (search-forward ";" (line-end-position) t)
+                                             (line-end-position))))
+                          (re-search-backward (concat ";\\|" cg-kw-re) nil 'noerror))))
+            (when kw-pos
+              (let* ((kw (match-string-no-properties 0)))
+                (if (and (not (equal kw ";"))
+                         (> origin (line-end-position)))
+                    cg-indentation
+                  0)))))
+      (setq case-fold-search old-case-fold-search))))
 
 (defun cg-indent-line ()
-  "Indent the current line. Very simple indentation: lines with
-keywords from `cg-kw-list' get zero indentation, others get one
-indentation."
+  "Indent the current line.
+
+Very simple indentation: lines with keywords from `cg-kw-list'
+get zero indentation, others get one indentation."
   (interactive)
   (let ((indent (cg-calculate-indent))
         (pos (- (point-max) (point))))
@@ -427,15 +502,16 @@ indentation."
 (defvar cg--goto-history nil)
 
 (defun cg-permute (input)
-  "From http://www.emacswiki.org/emacs/StringPermutations"
-  (require 'cl)	; TODO: (require 'cl-lib) for whole file when 24.3 in distros
+  "Permute INPUT list.
+
+From http://www.emacswiki.org/emacs/StringPermutations"
   (if (null input)
       (list input)
-    (mapcan (lambda (elt)
-              (mapcan (lambda (p)
-                        (list (cons elt p)))
-                      (cg-permute (remove* elt input :count 1))))
-            input)))
+    (cl-mapcan (lambda (elt)
+                 (cl-mapcan (lambda (p)
+                              (list (cons elt p)))
+                            (cg-permute (cl-remove elt input :count 1))))
+               input)))
 
 (defun cg-read-arg (prompt history &optional default)
   (let* ((default (or default (car history)))
@@ -484,16 +560,17 @@ etc."
       (setq regexp-history tmp))))
 
 (defun cg-goto-rule (&optional input)
-  "Go to the line number of the rule described by `input', where
-`input' is the rule info from vislcg3 --trace.  E.g. if `input'
-is \"SELECT:1022:rulename\", go to the rule on line number
-1022. Interactively, use a prefix argument to paste `input'
-manually, otherwise this function uses the most recently copied
-line in the X clipboard.
+  "Go to the line number of the rule described by INPUT.
+
+INPUT is the rule info from vislcg3 --trace; e.g. if INPUT is
+\"SELECT:1022:rulename\", go to the rule on line number 1022.
+Interactively, use a prefix argument to paste INPUT manually,
+otherwise this function uses the most recently copied line in the
+X clipboard.
 
 This makes switching between the terminal and the file slightly
-faster (since double-clicking the rule info -- in Konsole at
-least -- selects the whole string \"SELECT:1022:rulename\")."
+faster (since double-clicking the rule info in most terminals will
+select the whole string \"SELECT:1022:rulename\")."
   (interactive (list (when current-prefix-arg
                        (cg-read-arg "Paste rule info from --trace here: "
                                     cg--goto-history))))
@@ -545,11 +622,13 @@ to.")
 (defcustom cg-check-do-cache t
   "If non-nil, `cg-check' caches the output of `cg-pre-pipe' (the
 cache is emptied whenever you make a change in the input buffer,
-or call `cg-check' from another CG file).")
+or call `cg-check' from another CG file)."
+  :group 'cg
+  :type 'bool)
 
 (defvar cg--check-cache-buffer nil "See `cg-check-do-cache'.")
 
-(defun cg-input-mode-bork-cache (from to len)
+(defun cg-input-mode-bork-cache (_from _to _len)
   "Since `cg-check' will not reuse a cache unless `cg--file' and
 `cg--cache-in' match."
   (when cg--check-cache-buffer
@@ -601,29 +680,30 @@ from, otherwise all CG buffers share one input buffer."
 (defconst cg-output-regexp-alist
   `((,(format "%s:\\([^ \n\t:]+\\)\\(?::[^ \n\t]+\\)?" cg-kw-rule-re)
      ,#'cg-get-file 1 nil 1)
-    ("^Warning: .*?line \\([0-9]+\\)"
-     ,#'cg-get-file 1 nil 1)
-    ("^Warning: .*"
+    ("^\\([^:]*: \\)?Warning: .*?line \\([0-9]+\\).*"
+     ,#'cg-get-file 2 nil 1)
+    ("^\\([^:]*: \\)?Warning: .*"
      ,#'cg-get-file nil nil 1)
-    ("^Error: .*?line \\([0-9]+\\)"
-     ,#'cg-get-file 1 nil 2)
-    ("^Error: .*"
+    ("^\\([^:]*: \\)?Error: .*?line \\([0-9]+\\).*"
+     ,#'cg-get-file 2 nil 2)
+    ("^\\([^:]*: \\)?Error: .*"
      ,#'cg-get-file nil nil 2)
-    (".*?line \\([0-9]+\\)"		; some error messages span several lines
+    (".*?line \\([0-9]+\\).*"		; some error messages span several lines
      ,#'cg-get-file 1 nil 2))
-  "Regexp used to match vislcg3 --trace hits. See
-`compilation-error-regexp-alist'.")
+  "Regexp used to match vislcg3 --trace hits.
+See `compilation-error-regexp-alist'.")
 ;; TODO: highlight strings and @'s and #1->0's in cg-output-mode ?
 
 ;;;###autoload
 (defcustom cg-output-setup-hook nil
-  "List of hook functions run by `cg-output-process-setup' (see
-`run-hooks')."
+  "List of hook functions run by `cg-output-process-setup'.
+See `run-hooks'."
   :type 'hook)
 
 (defun cg-output-process-setup ()
-  "Runs `cg-output-setup-hook' for `cg-check'. That hook is
-useful for doing things like
+  "Run `cg-output-setup-hook' for `cg-check'.
+
+That hook is useful for doing things like
  (setenv \"PATH\" (concat \"~/local/stuff\" (getenv \"PATH\")))"
   (run-hooks 'cg-output-setup-hook))
 
@@ -637,7 +717,7 @@ useful for doing things like
   "Face name to use for lemmas in cg-output.")
 
 (defvar cg-output-mapping-face 'bold
-  "Face name to use for mapping tags in cg-output")
+  "Face name to use for mapping tags in cg-output.")
 
 (defvar cg-output-mode-font-lock-keywords
   '(("^;\\(?:[^:]* \\)"
@@ -725,21 +805,23 @@ runs."
     (overlay-put o 'isearch-open-invisible 'cg-output-remove-overlay)))
 
 (defun cg-output-show-all ()
-  "Undoes the effect of `cg-output-hide-analyses'."
+  "Undo the effect of `cg-output-hide-analyses'."
   (interactive)
   (setq cg--output-hiding-analyses nil)
   (remove-overlays (point-min) (point-max) 'invisible 'cg-output))
 
 (defun cg-output-hide-analyses ()
-  "Hides all analyses, turning the CG format back into input
-text (more or less). You can still isearch through the text for
-tags, REMOVE/SELECT keywords etc.
+  "Hide all analyses.
+
+This turns the CG format back into input text (more or less).
+You can still isearch through the text for tags, REMOVE/SELECT
+keywords etc.
 
 Call `cg-output-set-unhide' to set a regex which will be exempt
-from hiding. Call `cg-output-show-all' to turn off all hiding."
+from hiding.  Call `cg-output-show-all' to turn off all hiding."
   (interactive)
   (setq cg--output-hiding-analyses t)
-  (lexical-let (last)
+  (let (prev)
     (save-excursion
       (goto-char (point-min))
       (while (re-search-forward "^\"<.*>\"" nil 'noerror)
@@ -747,14 +829,14 @@ from hiding. Call `cg-output-show-all' to turn off all hiding."
 	      (line-end (match-end 0)))
 	  (cg-output-hide-region line-beg (+ line-beg 2)) ; "<
 	  (cg-output-hide-region (- line-end 2) line-end) ; >"
-	  (when last
-	    (if (save-excursion (re-search-backward cg-sent-tag last 'noerror))
-		(cg-output-hide-region last (- line-beg 1))	; show newline
-	      (cg-output-hide-region last line-beg))) ; hide newline too
-	  (setq last line-end)))
-      (goto-char last)
+	  (when prev
+	    (if (save-excursion (re-search-backward cg-sent-tag prev 'noerror))
+		(cg-output-hide-region prev (- line-beg 1))	; show newline
+	      (cg-output-hide-region prev line-beg))) ; hide newline too
+	  (setq prev line-end)))
+      (goto-char prev)
       (when (re-search-forward "^[^\t\"]" nil 'noerror)
-	(cg-output-hide-region last (match-beginning 0)))))
+	(cg-output-hide-region prev (match-beginning 0)))))
 
   (when cg-output-unhide-regex
     (cg-output-unhide-some cg-output-unhide-regex)))
@@ -770,8 +852,11 @@ from hiding. Call `cg-output-show-all' to turn off all hiding."
 	    (overlays-at (match-beginning 0))))))
 
 (defun cg-output-set-unhide (needle)
-  "Set some exeption to `cg-output-hide-analyses'. This is saved
-and reused whenever `cg-output-hide-analyses' is called."
+  "Set some exeption to `cg-output-hide-analyses'.
+
+If NEEDLE is the empty string, hide all analyses.
+This is saved and reused whenever `cg-output-hide-analyses' is
+called."
   (interactive (list (cg-read-arg
 		      "Regex to unhide, or empty to hide all"
 		      cg--output-unhide-history
@@ -784,8 +869,8 @@ and reused whenever `cg-output-hide-analyses' is called."
 
 ;;; TODO:
 (defun cg-output-toggle-analyses ()
-  "Hide or show analyses from output. See
-`cg-output-hide-analyses'."
+  "Hide or show analyses from output.
+See `cg-output-hide-analyses'."
   (interactive)
   (if cg--output-hiding-analyses
       (cg-output-show-all)
@@ -795,17 +880,18 @@ and reused whenever `cg-output-hide-analyses' is called."
 
 ;;;###autoload
 (defcustom cg-check-after-change nil
-  "If non-nil, run `cg-check' on grammar after each change to the
-buffer.")
+  "If non-nil, run `cg-check' on grammar after each change to the buffer."
+  :group 'cg
+  :type 'bool)
 
 ;;;###autoload
 (defcustom cg-check-after-change-secs 1
-  "Minimum seconds between each `cg-check' after a change to a CG
-buffer (so 0 is after each change)."
+  "Minimum seconds between each `cg-check' after a change to a CG buffer.
+Use 0 to check immediately after each change."
   :type 'integer)
 
 (defvar cg--after-change-timer nil)
-(defun cg-after-change (from to len)
+(defun cg-after-change (_from _to _len)
   (when (and cg-check-after-change
              (not (member cg--after-change-timer timer-list)))
     (setq
@@ -854,7 +940,7 @@ buffer (so 0 is after each change)."
 			'cg-output-buffer-name))))
 
 (defun cg-end-process (proc &optional string)
-  "End `proc', optionally first sending in `string'."
+  "End PROC, optionally first sending in STRING."
   (when string
     (process-send-string proc string))
   (process-send-string proc "\n")
@@ -865,14 +951,14 @@ buffer (so 0 is after each change)."
 in case you haven't saved yet).
 
 If you've set `cg-pre-pipe', input will first be sent through
-that. Set your test input sentence(s) with `cg-edit-input'. If
-you want to send a whole file instead, just set `cg-pre-pipe' to
+that.  Set your test input sentence(s) with `cg-edit-input'.
+If you want to send a whole file instead, just set `cg-pre-pipe' to
 something like
 \"zcat corpus.gz | lt-proc analyser.bin | cg-conv\".
 
 Similarly, `cg-post-pipe' is run on output."
   (interactive)
-  (lexical-let*
+  (let*
       ((file (buffer-file-name))
        (tmp (make-temp-file "cg."))
        ;; Run in a separate process buffer from cmd and post-pipe:
@@ -908,16 +994,16 @@ Similarly, `cg-post-pipe' is run on output."
         (with-current-buffer cg--check-cache-buffer
           (cg-end-process (get-buffer-process out) (buffer-string)))
 
-      (lexical-let ((cg-proc (get-buffer-process out))
-                    (pre-proc (start-process "cg-pre-pipe" "*cg-pre-pipe-output*"
-                                             "/bin/bash" "-c" pre-pipe))
-                    (cache-buffer (cg-pristine-cache-buffer file in pre-pipe)))
-        (set-process-filter pre-proc (lambda (pre-proc string)
+      (let ((cg-proc (get-buffer-process out))
+            (pre-proc (start-process "cg-pre-pipe" "*cg-pre-pipe-output*"
+                                     "/bin/bash" "-c" pre-pipe))
+            (cache-buffer (cg-pristine-cache-buffer file in pre-pipe)))
+        (set-process-filter pre-proc (lambda (_pre-proc string)
                                        (with-current-buffer cache-buffer
                                          (insert string))
                                        (when (eq (process-status cg-proc) 'run)
                                          (process-send-string cg-proc string))))
-        (set-process-sentinel pre-proc (lambda (pre-proc string)
+        (set-process-sentinel pre-proc (lambda (_pre-proc _string)
                                          (when (eq (process-status cg-proc) 'run)
                                            (cg-end-process cg-proc))))
         (with-current-buffer in
@@ -925,7 +1011,7 @@ Similarly, `cg-post-pipe' is run on output."
 
     (display-buffer out)))
 
-(defun cg-check-finish-function (buffer change)
+(defun cg-check-finish-function (buffer _change)
   ;; Note: this makes `recompile' not work, which is why `g' is
   ;; rebound in `cg-output-mode'
   (let ((w (get-buffer-window buffer)))
@@ -969,10 +1055,13 @@ Similarly, `cg-post-pipe' is run on output."
 
 ;;; Keybindings ---------------------------------------------------------------
 (define-key cg-mode-map (kbd "C-c C-o") #'cg-occur-list)
-(define-key cg-mode-map (kbd "C-c g") #'cg-goto-rule)
+(define-key cg-mode-map (kbd "C-c C-r") #'cg-goto-rule)
 (define-key cg-mode-map (kbd "C-c C-c") #'cg-check)
 (define-key cg-mode-map (kbd "C-c C-i") #'cg-edit-input)
-(define-key cg-mode-map (kbd "C-c c") #'cg-toggle-check-after-change)
+(define-key cg-mode-map (kbd "C-c M-c") #'cg-toggle-check-after-change)
+(define-key cg-mode-map (kbd "C-;") #'cg-comment-or-uncomment-rule)
+(define-key cg-mode-map (kbd "M-#") #'cg-comment-or-uncomment-rule)
+
 (define-key cg-output-mode-map (kbd "C-c C-i") #'cg-back-to-file-and-edit-input)
 (define-key cg-output-mode-map (kbd "i") #'cg-back-to-file-and-edit-input)
 (define-key cg-output-mode-map (kbd "g") #'cg-back-to-file-and-check)
