@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2016, GrammarSoft ApS
+* Copyright (C) 2007-2017, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -42,16 +42,13 @@ ApertiumApplicator::ApertiumApplicator(UFILE *ux_err)
 	fgetc_error = U_ZERO_ERROR;
 }
 
-
-bool ApertiumApplicator::getNullFlush() {
-	return nullFlush;
-}
-
 void ApertiumApplicator::setNullFlush(bool pNullFlush) {
 	nullFlush = pNullFlush;
 }
 
 UChar ApertiumApplicator::u_fgetc_wrapper(istream& input) {
+	UChar rv = U_EOF;
+
 	if (runningWithNullFlush) {
 		if (!fgetc_converter) {
 			fgetc_error = U_ZERO_ERROR;
@@ -83,11 +80,17 @@ UChar ApertiumApplicator::u_fgetc_wrapper(istream& input) {
 		if (fgetc_outputbuf[0] == 0xFFFD && input.eof()) {
 			return U_EOF;
 		}
-		return fgetc_outputbuf[0];
+		rv = fgetc_outputbuf[0];
 	}
 	else {
-		return input.getc();
+		rv = input.getc();
 	}
+
+	if (ISNL(rv)) {
+		++numLines;
+	}
+
+	return rv;
 }
 
 
@@ -107,7 +110,7 @@ void ApertiumApplicator::runGrammarOnTextWrapperNullFlush(istream& input, UFILE
  */
 
 void ApertiumApplicator::runGrammarOnText(istream& input, UFILE *output) {
-	if (getNullFlush()) {
+	if (nullFlush) {
 		runGrammarOnTextWrapperNullFlush(input, output);
 		return;
 	}
@@ -217,8 +220,8 @@ void ApertiumApplicator::runGrammarOnText(istream& input, UFILE *output) {
 			}
 			if (cCohort && cSWindow->cohorts.size() >= soft_limit && grammar->soft_delimiters && doesSetMatchCohortNormal(*cCohort, grammar->soft_delimiters->number)) {
 				// ie. we've read some cohorts
-				foreach (iter, cCohort->readings) {
-					addTagToReading(**iter, endtag);
+				for (auto iter : cCohort->readings) {
+					addTagToReading(*iter, endtag);
 				}
 
 				cSWindow->appendCohort(cCohort);
@@ -229,11 +232,11 @@ void ApertiumApplicator::runGrammarOnText(istream& input, UFILE *output) {
 			} // end >= soft_limit
 			if (cCohort && (cSWindow->cohorts.size() >= hard_limit || (grammar->delimiters && doesSetMatchCohortNormal(*cCohort, grammar->delimiters->number)))) {
 				if (!is_conv && cSWindow->cohorts.size() >= hard_limit) {
-					u_fprintf(ux_stderr, "Warning: Hard limit of %u cohorts reached at line %u - forcing break.\n", hard_limit, numLines);
+					u_fprintf(ux_stderr, "Warning: Hard limit of %u cohorts reached at cohort %u on line %u - forcing break.\n", hard_limit, numCohorts, numLines);
 					u_fflush(ux_stderr);
 				}
-				foreach (iter, cCohort->readings) {
-					addTagToReading(**iter, endtag);
+				for (auto iter : cCohort->readings) {
+					addTagToReading(*iter, endtag);
 				}
 
 				cSWindow->appendCohort(cCohort);
@@ -405,11 +408,10 @@ void ApertiumApplicator::runGrammarOnText(istream& input, UFILE *output) {
 			} // end while not $
 
 			if (!cReading->baseform) {
-				u_fprintf(ux_stderr, "Warning: Line %u had no valid baseform.\n", numLines);
+				u_fprintf(ux_stderr, "Warning: Cohort %u on line %u had no valid baseform.\n", numCohorts, numLines);
 				u_fflush(ux_stderr);
 			}
 		} // end reading
-		numLines++;
 	} // end input loop
 
 	if (!firstblank.empty()) {
@@ -423,8 +425,8 @@ void ApertiumApplicator::runGrammarOnText(istream& input, UFILE *output) {
 		if (cCohort->readings.empty()) {
 			initEmptyCohort(*cCohort);
 		}
-		foreach (iter, cCohort->readings) {
-			addTagToReading(**iter, endtag);
+		for (auto iter : cCohort->readings) {
+			addTagToReading(*iter, endtag);
 		}
 		cReading = 0;
 		cCohort = 0;
@@ -692,7 +694,7 @@ void ApertiumApplicator::printReading(Reading *reading, UFILE *output) {
 
 	if (reading->baseform) {
 		// Lop off the initial and final '"' characters
-		UnicodeString bf(single_tags[reading->baseform]->tag.c_str() + 1, single_tags[reading->baseform]->tag.length() - 2);
+		UnicodeString bf(single_tags[reading->baseform]->tag.c_str() + 1, single_tags[reading->baseform]->tag.size() - 2);
 
 		if (wordform_case && !reading->next) {
 			// Use surface/wordform case, eg. if lt-proc
@@ -700,7 +702,7 @@ void ApertiumApplicator::printReading(Reading *reading, UFILE *output) {
 			// dictionary case on lemma/basefrom)
 			// Lop off the initial and final '"<>"' characters
 			// ToDo: A copy does not need to be made here - use pointer offsets
-			UnicodeString wf(reading->parent->wordform->tag.c_str() + 2, reading->parent->wordform->tag.length() - 4);
+			UnicodeString wf(reading->parent->wordform->tag.c_str() + 2, reading->parent->wordform->tag.size() - 4);
 
 			int first = 0; // first occurrence of a lowercase character in baseform
 			for (; first < bf.length(); ++first) {
@@ -787,9 +789,9 @@ void ApertiumApplicator::printReading(Reading *reading, UFILE *output) {
 	}
 
 	if (trace) {
-		foreach (iter_hb, reading->hit_by) {
+		for (auto iter_hb : reading->hit_by) {
 			u_fputc('<', output);
-			printTrace(output, *iter_hb);
+			printTrace(output, iter_hb);
 			u_fputc('>', output);
 		}
 	}
@@ -818,7 +820,7 @@ void ApertiumApplicator::printSingleWindow(SingleWindow *window, UFILE *output)
 		if (print_word_forms == true) {
 			// Lop off the initial and final '"' characters
 			// ToDo: A copy does not need to be made here - use pointer offsets
-			UnicodeString wf(cohort->wordform->tag.c_str() + 2, cohort->wordform->tag.length() - 4);
+			UnicodeString wf(cohort->wordform->tag.c_str() + 2, cohort->wordform->tag.size() - 4);
 			UString wf_escaped;
 			for (int i = 0; i < wf.length(); ++i) {
 				if (wf[i] == '^' || wf[i] == '\\' || wf[i] == '/' || wf[i] == '$' || wf[i] == '[' || wf[i] == ']' || wf[i] == '{' || wf[i] == '}' || wf[i] == '<' || wf[i] == '>') {
@@ -830,11 +832,11 @@ void ApertiumApplicator::printSingleWindow(SingleWindow *window, UFILE *output)
 
 			// Print the static reading tags
 			if (cohort->wread) {
-				foreach (tter, cohort->wread->tags_list) {
-					if (*tter == cohort->wordform->hash) {
+				for (auto tter : cohort->wread->tags_list) {
+					if (tter == cohort->wordform->hash) {
 						continue;
 					}
-					const Tag *tag = single_tags[*tter];
+					const Tag *tag = single_tags[tter];
 					u_fprintf(output, "<%S>", tag->tag.c_str());
 				}
 			}
@@ -843,7 +845,7 @@ void ApertiumApplicator::printSingleWindow(SingleWindow *window, UFILE *output)
 		bool need_slash = print_word_forms;
 
 		//Tag::printTagRaw(output, single_tags[cohort->wordform]);
-		boost_foreach (Reading *reading, cohort->readings) {
+		for (auto reading : cohort->readings) {
 			if (need_slash) {
 				u_fprintf(output, "/");
 			}
@@ -859,7 +861,7 @@ void ApertiumApplicator::printSingleWindow(SingleWindow *window, UFILE *output)
 
 		if (trace) {
 			const UChar not_sign = L'\u00AC';
-			boost_foreach (Reading *reading, cohort->delayed) {
+			for (auto reading : cohort->delayed) {
 				if (need_slash) {
 					u_fprintf(output, "/%C", not_sign);
 				}
@@ -869,7 +871,7 @@ void ApertiumApplicator::printSingleWindow(SingleWindow *window, UFILE *output)
 				}
 				printReading(reading, output);
 			}
-			boost_foreach (Reading *reading, cohort->deleted) {
+			for (auto reading : cohort->deleted) {
 				if (need_slash) {
 					u_fprintf(output, "/%C", not_sign);
 				}
@@ -899,20 +901,20 @@ void ApertiumApplicator::mergeMappings(Cohort& cohort) {
 	// foo<N><Sg><Acc><@←SUBJ>/foo<N><Sg><Acc><@←OBJ>
 	// => foo<N><Sg><Acc><@←SUBJ>/foo<N><Sg><Acc><@←OBJ>
 	std::map<uint32_t, ReadingList> mlist;
-	foreach (iter, cohort.readings) {
-		Reading *r = *iter;
+	for (auto iter : cohort.readings) {
+		Reading *r = iter;
 		uint32_t hp = r->hash; // instead of hash_plain, which doesn't include mapping tags
 		if (trace) {
-			foreach (iter_hb, r->hit_by) {
-				hp = hash_value(*iter_hb, hp);
+			for (auto iter_hb : r->hit_by) {
+				hp = hash_value(iter_hb, hp);
 			}
 		}
 		Reading *sub = r->next;
 		while (sub) {
 			hp = hash_value(sub->hash, hp);
 			if (trace) {
-				foreach (iter_hb, sub->hit_by) {
-					hp = hash_value(*iter_hb, hp);
+				for (auto iter_hb : sub->hit_by) {
+					hp = hash_value(iter_hb, hp);
 				}
 			}
 			sub = sub->next;
@@ -934,8 +936,8 @@ void ApertiumApplicator::mergeMappings(Cohort& cohort) {
 		order.push_back(clist.front());
 
 		clist.erase(clist.begin());
-		foreach (cit, clist) {
-			free_reading(*cit);
+		for (auto cit : clist) {
+			free_reading(cit);
 		}
 	}
 
