@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2015, GrammarSoft ApS
+* Copyright (C) 2007-2017, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -25,50 +25,51 @@
 
 namespace CG3 {
 
-Grammar::Grammar() :
-ux_stderr(0),
-ux_stdout(0),
-has_dep(false),
-has_relations(false),
-has_encl_final(false),
-is_binary(false),
-sub_readings_ltr(false),
-grammar_size(0),
-mapping_prefix('@'),
-lines(0),
-verbosity_level(0),
-total_time(0),
-rules_any(0),
-sets_any(0),
-delimiters(0),
-soft_delimiters(0),
-tag_any(0)
+Grammar::Grammar()
+  : ux_stderr(0)
+  , ux_stdout(0)
+  , has_dep(false)
+  , has_bag_of_tags(false)
+  , has_relations(false)
+  , has_encl_final(false)
+  , is_binary(false)
+  , sub_readings_ltr(false)
+  , grammar_size(0)
+  , mapping_prefix('@')
+  , lines(0)
+  , verbosity_level(0)
+  , total_time(0)
+  , rules_any(0)
+  , sets_any(0)
+  , delimiters(0)
+  , soft_delimiters(0)
+  , tag_any(0)
 {
 	// Nothing in the actual body...
 }
 
 Grammar::~Grammar() {
-	foreach (std::vector<Set*>, sets_list, iter_set, iter_set_end) {
-		destroySet(*iter_set);
+	for (auto iter_set : sets_list) {
+		destroySet(iter_set);
 	}
 
-	foreach (SetSet, sets_all, rsets, rsets_end) {
-		delete *rsets;
+	for (auto rsets : sets_all) {
+		delete rsets;
 	}
-	
+
 	Taguint32HashMap::iterator iter_stag;
-	for (iter_stag = single_tags.begin() ; iter_stag != single_tags.end() ; ++iter_stag) {
+	for (iter_stag = single_tags.begin(); iter_stag != single_tags.end(); ++iter_stag) {
 		if (iter_stag->second) {
 			delete iter_stag->second;
 		}
 	}
 
-	foreach (RuleVector, rule_by_number, iter_rules, iter_rules_end) {
-		delete *iter_rules;
+	for (auto iter_rules : rule_by_number) {
+		delete iter_rules;
 	}
 
-	for (BOOST_AUTO(cntx, contexts.begin()); cntx != contexts.end(); ++cntx) {
-		delete cntx->second;
+	for (auto cntx : contexts) {
+		delete cntx.second;
 	}
 }
 
@@ -83,10 +84,10 @@ void Grammar::addSet(Set *& to) {
 		u_fprintf(ux_stderr, "Warning: Set name %S looks like a misattempt of template usage on line %u.\n", to->name.c_str(), to->line);
 	}
 
-	if (!to->sets.empty() && !(to->type & (ST_TAG_UNIFY|ST_CHILD_UNIFY|ST_SET_UNIFY))) {
+	if (!to->sets.empty() && !(to->type & (ST_TAG_UNIFY | ST_CHILD_UNIFY | ST_SET_UNIFY))) {
 		bool all_tags = true;
-		for (size_t i=0 ; i<to->sets.size() ; ++i) {
-			if (i > 0 && to->set_ops[i-1] != S_OR) {
+		for (size_t i = 0; i < to->sets.size(); ++i) {
+			if (i > 0 && to->set_ops[i - 1] != S_OR) {
 				all_tags = false;
 				break;
 			}
@@ -106,7 +107,7 @@ void Grammar::addSet(Set *& to) {
 		}
 
 		if (all_tags) {
-			for (size_t i=0 ; i<to->sets.size() ; ++i) {
+			for (size_t i = 0; i < to->sets.size(); ++i) {
 				Set *s = getSet(to->sets[i]);
 				maybe_used_sets.insert(s);
 				TagVector tv = trie_getTagList(s->getNonEmpty());
@@ -115,7 +116,7 @@ void Grammar::addSet(Set *& to) {
 				}
 				else {
 					bool special = false;
-					boost_foreach (Tag *tag, tv) {
+					for (auto tag : tv) {
 						if (tag->type & T_SPECIAL) {
 							special = true;
 							break;
@@ -160,8 +161,8 @@ void Grammar::addSet(Set *& to) {
 		positive->trie.swap(to->trie);
 		positive->trie_special.swap(to->trie_special);
 
-		boost_foreach (Tag *iter, to->ff_tags) {
-			BOOST_AUTO(pit, positive->trie_special.find(iter));
+		for (auto iter : to->ff_tags) {
+			auto pit = positive->trie_special.find(iter);
 			if (pit != positive->trie_special.end()) {
 				if (pit->second.terminal) {
 					if (pit->second.trie) {
@@ -195,7 +196,7 @@ void Grammar::addSet(Set *& to) {
 	}
 
 	uint32_t chash = to->rehash();
-	for (; to->name[0] != '_' || to->name[1] != 'G' || to->name[2] != '_' ;) {
+	for (; to->name[0] != '_' || to->name[1] != 'G' || to->name[2] != '_';) {
 		uint32_t nhash = hash_value(to->name.c_str());
 		if (sets_by_name.find(nhash) != sets_by_name.end()) {
 			Set *a = sets_by_contents.find(sets_by_name.find(nhash)->second)->second;
@@ -217,14 +218,14 @@ void Grammar::addSet(Set *& to) {
 				CG3Quit(1);
 			}
 			else {
-				for (uint32_t seed=0 ; seed<1000 ; ++seed) {
-					if (sets_by_name.find(nhash+seed) == sets_by_name.end()) {
+				for (uint32_t seed = 0; seed < 1000; ++seed) {
+					if (sets_by_name.find(nhash + seed) == sets_by_name.end()) {
 						if (verbosity_level > 0 && (to->name[0] != '_' || to->name[1] != 'G' || to->name[2] != '_')) {
 							u_fprintf(ux_stderr, "Warning: Set %S got hash seed %u.\n", to->name.c_str(), seed);
 							u_fflush(ux_stderr);
 						}
 						set_name_seeds[to->name] = seed;
-						sets_by_name[nhash+seed] = chash;
+						sets_by_name[nhash + seed] = chash;
 						break;
 					}
 				}
@@ -240,9 +241,7 @@ void Grammar::addSet(Set *& to) {
 		if (a != to) {
 			a->reindex(*this);
 			to->reindex(*this);
-			if ((a->type & (ST_SPECIAL|ST_TAG_UNIFY|ST_CHILD_UNIFY|ST_SET_UNIFY)) != (to->type & (ST_SPECIAL|ST_TAG_UNIFY|ST_CHILD_UNIFY|ST_SET_UNIFY))
-			|| a->set_ops.size() != to->set_ops.size() || a->sets.size() != to->sets.size()
-			|| a->trie.size() != to->trie.size() || a->trie_special.size() != to->trie_special.size()) {
+			if ((a->type & (ST_SPECIAL | ST_TAG_UNIFY | ST_CHILD_UNIFY | ST_SET_UNIFY)) != (to->type & (ST_SPECIAL | ST_TAG_UNIFY | ST_CHILD_UNIFY | ST_SET_UNIFY)) || a->set_ops.size() != to->set_ops.size() || a->sets.size() != to->sets.size() || a->trie.size() != to->trie.size() || a->trie_special.size() != to->trie_special.size()) {
 				u_fprintf(ux_stderr, "Error: Content hash collision between set %S on line %u and %S on line %u!\n", a->name.c_str(), a->line, to->name.c_str(), to->line);
 				CG3Quit(1);
 			}
@@ -290,12 +289,12 @@ void Grammar::addSetToList(Set *s) {
 	if (s->number == 0) {
 		if (sets_list.empty() || sets_list[0] != s) {
 			if (!s->sets.empty()) {
-				foreach (uint32Vector, s->sets, sit, sit_end) {
-					addSetToList(getSet(*sit));
+				for (auto sit : s->sets) {
+					addSetToList(getSet(sit));
 				}
 			}
 			sets_list.push_back(s);
-			s->number = (uint32_t)sets_list.size()-1;
+			s->number = (uint32_t)sets_list.size() - 1;
 		}
 	}
 }
@@ -315,7 +314,7 @@ uint32_t Grammar::removeNumericTags(uint32_t s) {
 	Set *set = getSet(s);
 	if (!set->sets.empty()) {
 		bool did = false;
-		BOOST_AUTO(sets, set->sets);
+		auto sets = set->sets;
 		for (size_t i = 0; i < sets.size(); ++i) {
 			uint32_t ns = removeNumericTags(sets[i]);
 			if (ns == 0) {
@@ -347,13 +346,13 @@ uint32_t Grammar::removeNumericTags(uint32_t s) {
 		bool did = false;
 		std::map<TagVector, bool> ntags;
 		TagVector tags;
-		const trie_t* tries[2] = { &set->trie, &set->trie_special };
+		const trie_t *tries[2] = { &set->trie, &set->trie_special };
 		for (size_t i = 0; i < 2; ++i) {
 			if (tries[i]->empty()) {
 				continue;
 			}
-			BOOST_AUTO(ctags, trie_getTags(*tries[i]));
-			for (BOOST_AUTO(it, ctags.begin()); it != ctags.end(); ++it) {
+			auto ctags = trie_getTags(*tries[i]);
+			for (auto it = ctags.begin(); it != ctags.end(); ++it) {
 				bool special = false;
 				tags.clear();
 				fill_tagvector(*it, tags, did, special);
@@ -391,7 +390,7 @@ uint32_t Grammar::removeNumericTags(uint32_t s) {
 			ns->name += 'B';
 			ns->name += '_';
 
-			for (BOOST_AUTO(it, ntags.begin()); it != ntags.end(); ++it) {
+			for (auto it = ntags.begin(); it != ntags.end(); ++it) {
 				if (it->second) {
 					if (it->first.size() == 1 && (it->first[0]->type & T_FAILFAST)) {
 						ns->ff_tags.insert(it->first[0]);
@@ -411,6 +410,17 @@ uint32_t Grammar::removeNumericTags(uint32_t s) {
 	return set->hash;
 }
 
+void Grammar::getTags(const Set& set, std::set<TagVector>& rv) {
+	// ToDo: getTags() ought to account for other operators than OR
+	for (auto s : set.sets) {
+		getTags(*getSet(s), rv);
+	}
+	TagVector tv;
+	trie_getTags(set.trie, rv, tv);
+	tv.clear();
+	trie_getTags(set.trie_special, rv, tv);
+}
+
 Rule *Grammar::allocateRule() {
 	return new Rule;
 }
@@ -512,14 +522,14 @@ ContextualTest *Grammar::addContextualTest(ContextualTest *t) {
 	t->rehash();
 
 	t->linked = addContextualTest(t->linked);
-	boost_foreach (ContextualTest *& it, t->ors) {
+	for (auto& it : t->ors) {
 		it = addContextualTest(it);
 	}
 
 	for (uint32_t seed = 0; seed < 1000; ++seed) {
-		contexts_t::iterator cit = contexts.find(t->hash+seed);
+		contexts_t::iterator cit = contexts.find(t->hash + seed);
 		if (cit == contexts.end()) {
-			contexts[t->hash+seed] = t;
+			contexts[t->hash + seed] = t;
 			t->hash += seed;
 			t->seed = seed;
 			if (verbosity_level > 1 && seed) {
@@ -567,47 +577,46 @@ void Grammar::addAnchor(const UChar *to, uint32_t at, bool primary) {
 
 void Grammar::resetStatistics() {
 	total_time = 0;
-	for (uint32_t j=0;j<rules.size();j++) {
+	for (uint32_t j = 0; j < rules.size(); j++) {
 		rules[j]->resetStatistics();
 	}
 }
 
 void Grammar::renameAllRules() {
-	foreach (RuleVector, rule_by_number, iter_rule, iter_rule_end) {
-		Rule *r = *iter_rule;
+	for (auto r : rule_by_number) {
 		gbuffers[0][0] = 0;
 		u_sprintf(&gbuffers[0][0], "L%u", r->line);
 		r->setName(&gbuffers[0][0]);
 	}
-};
+}
 
 void Grammar::reindex(bool unused_sets, bool used_tags) {
-	foreach (Setuint32HashMap, sets_by_contents, dset, dset_end) {
-		if (dset->second->number == std::numeric_limits<uint32_t>::max()) {
-			dset->second->type |= ST_USED;
+	for (auto dset : sets_by_contents) {
+		if (dset.second->number == std::numeric_limits<uint32_t>::max()) {
+			dset.second->type |= ST_USED;
 			continue;
 		}
-		if (!(dset->second->type & ST_STATIC)) {
-			dset->second->type &= ~ST_USED;
+		if (!(dset.second->type & ST_STATIC)) {
+			dset.second->type &= ~ST_USED;
 		}
-		dset->second->number = 0;
+		dset.second->number = 0;
 	}
 
-	foreach (static_sets_t, static_sets, sset, sset_end) {
-		uint32_t sh = hash_value(*sset);
+	for (auto sset : static_sets) {
+		uint32_t sh = hash_value(sset);
 		if (set_alias.find(sh) != set_alias.end()) {
-			u_fprintf(ux_stderr, "Error: Static set %S is an alias; only real sets may be made static!\n", (*sset).c_str());
+			u_fprintf(ux_stderr, "Error: Static set %S is an alias; only real sets may be made static!\n", sset.c_str());
 			CG3Quit(1);
 		}
 		Set *s = getSet(sh);
 		if (!s) {
 			if (verbosity_level > 0) {
-				u_fprintf(ux_stderr, "Warning: Set %S was not defined, so cannot make it static.\n", (*sset).c_str());
+				u_fprintf(ux_stderr, "Warning: Set %S was not defined, so cannot make it static.\n", sset.c_str());
 			}
 			continue;
 		}
-		if (s->name != *sset) {
-			s->setName(*sset);
+		if (s->name != sset) {
+			s->setName(sset);
 		}
 		s->markUsed(*this);
 		s->type |= ST_STATIC;
@@ -628,41 +637,41 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 	sets_any = 0;
 	rules_any = 0;
 
-	foreach (TagVector, single_tags_list, iter, iter_end) {
-		if ((*iter)->regexp && (*iter)->tag[0] != '"' && (*iter)->tag[0] != '<') {
-			regex_tags.insert((*iter)->regexp);
+	for (auto iter : single_tags_list) {
+		if (iter->regexp && iter->tag[0] != '"' && iter->tag[0] != '<') {
+			regex_tags.insert(iter->regexp);
 		}
-		if (((*iter)->type & T_CASE_INSENSITIVE) && (*iter)->tag[0] != '"' && (*iter)->tag[0] != '<') {
-			icase_tags.insert((*iter));
+		if ((iter->type & T_CASE_INSENSITIVE) && iter->tag[0] != '"' && iter->tag[0] != '<') {
+			icase_tags.insert(iter);
 		}
 		if (is_binary) {
 			continue;
 		}
-		if (!(*iter)->vs_sets) {
+		if (!iter->vs_sets) {
 			continue;
 		}
-		foreach (SetVector, *(*iter)->vs_sets, sit, sit_end) {
-			(*sit)->markUsed(*this);
+		for (auto sit : *iter->vs_sets) {
+			sit->markUsed(*this);
 		}
 	}
 
-	foreach (TagVector, single_tags_list, titer, titer_end) {
-		if ((*titer)->type & T_TEXTUAL) {
+	for (auto titer : single_tags_list) {
+		if (titer->type & T_TEXTUAL) {
 			continue;
 		}
-		foreach (Grammar::regex_tags_t, regex_tags, iter, iter_end) {
+		for (auto iter : regex_tags) {
 			UErrorCode status = U_ZERO_ERROR;
-			uregex_setText(*iter, (*titer)->tag.c_str(), (*titer)->tag.length(), &status);
+			uregex_setText(iter, titer->tag.c_str(), titer->tag.size(), &status);
 			if (status == U_ZERO_ERROR) {
-				if (uregex_find(*iter, -1, &status)) {
-					(*titer)->type |= T_TEXTUAL;
+				if (uregex_find(iter, -1, &status)) {
+					titer->type |= T_TEXTUAL;
 				}
 			}
 		}
-		foreach (Grammar::icase_tags_t, icase_tags, iter, iter_end) {
+		for (auto iter : icase_tags) {
 			UErrorCode status = U_ZERO_ERROR;
-			if (u_strCaseCompare((*titer)->tag.c_str(), (*titer)->tag.length(), (*iter)->tag.c_str(), (*iter)->tag.length(), U_FOLD_CASE_DEFAULT, &status) == 0) {
-				(*titer)->type |= T_TEXTUAL;
+			if (u_strCaseCompare(titer->tag.c_str(), titer->tag.size(), iter->tag.c_str(), iter->tag.size(), U_FOLD_CASE_DEFAULT, &status) == 0) {
+				titer->type |= T_TEXTUAL;
 			}
 			if (status != U_ZERO_ERROR) {
 				u_fprintf(ux_stderr, "Error: u_strCaseCompare() returned %s - cannot continue!\n", u_errorName(status));
@@ -671,47 +680,47 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 		}
 	}
 
-	for (BOOST_AUTO(it, parentheses.begin()); it != parentheses.end(); ++it) {
+	for (auto it = parentheses.begin(); it != parentheses.end(); ++it) {
 		single_tags[it->first]->markUsed();
 		single_tags[it->second]->markUsed();
 	}
 
-	for (BOOST_AUTO(it, preferred_targets.begin()); it != preferred_targets.end(); ++it) {
+	for (auto it = preferred_targets.begin(); it != preferred_targets.end(); ++it) {
 		single_tags[*it]->markUsed();
 	}
 
-	foreach (RuleVector, rule_by_number, iter_rule, iter_rule_end) {
-		if ((*iter_rule)->wordform) {
-			wf_rules.push_back(*iter_rule);
+	for (auto rule : rule_by_number) {
+		if (rule->wordform) {
+			wf_rules.push_back(rule);
 		}
 		if (is_binary) {
 			continue;
 		}
 		Set *s = 0;
-		s = getSet((*iter_rule)->target);
+		s = getSet(rule->target);
 		s->markUsed(*this);
-		if ((*iter_rule)->childset1) {
-			s = getSet((*iter_rule)->childset1);
+		if (rule->childset1) {
+			s = getSet(rule->childset1);
 			s->markUsed(*this);
 		}
-		if ((*iter_rule)->childset2) {
-			s = getSet((*iter_rule)->childset2);
+		if (rule->childset2) {
+			s = getSet(rule->childset2);
 			s->markUsed(*this);
 		}
-		if ((*iter_rule)->maplist) {
-			(*iter_rule)->maplist->markUsed(*this);
+		if (rule->maplist) {
+			rule->maplist->markUsed(*this);
 		}
-		if ((*iter_rule)->sublist) {
-			(*iter_rule)->sublist->markUsed(*this);
+		if (rule->sublist) {
+			rule->sublist->markUsed(*this);
 		}
-		if ((*iter_rule)->dep_target) {
-			(*iter_rule)->dep_target->markUsed(*this);
+		if (rule->dep_target) {
+			rule->dep_target->markUsed(*this);
 		}
-		foreach (ContextList, (*iter_rule)->tests, it, it_end) {
-			(*it)->markUsed(*this);
+		for (auto it : rule->tests) {
+			it->markUsed(*this);
 		}
-		foreach (ContextList, (*iter_rule)->dep_tests, it, it_end) {
-			(*it)->markUsed(*this);
+		for (auto it : rule->dep_tests) {
+			it->markUsed(*this);
 		}
 	}
 
@@ -724,7 +733,7 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 		}
 
 		contexts_t tosave;
-		for (BOOST_AUTO(cntx, contexts.begin()); cntx != contexts.end(); ++cntx) {
+		for (auto cntx = contexts.begin(); cntx != contexts.end(); ++cntx) {
 			if (cntx->second->is_used) {
 				tosave[cntx->first] = cntx->second;
 				continue;
@@ -736,10 +745,10 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 
 	if (unused_sets) {
 		u_fprintf(ux_stdout, "Unused sets:\n");
-		foreach (Setuint32HashMap, sets_by_contents, rset, rset_end) {
-			if (!(rset->second->type & ST_USED) && !rset->second->name.empty() && maybe_used_sets.count(rset->second) == 0) {
-				if (rset->second->name[0] != '_' || rset->second->name[1] != 'G' || rset->second->name[2] != '_') {
-					u_fprintf(ux_stdout, "Line %u set %S\n", rset->second->line, rset->second->name.c_str());
+		for (auto rset : sets_by_contents) {
+			if (!(rset.second->type & ST_USED) && !rset.second->name.empty() && maybe_used_sets.count(rset.second) == 0) {
+				if (rset.second->name[0] != '_' || rset.second->name[1] != 'G' || rset.second->name[2] != '_') {
+					u_fprintf(ux_stdout, "Line %u set %S\n", rset.second->line, rset.second->name.c_str());
 				}
 			}
 		}
@@ -749,14 +758,14 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 
 	// Stuff below this line is not optional...
 
-	foreach (Setuint32HashMap, sets_by_contents, tset, tset_end) {
-		if (tset->second->type & ST_USED) {
-			addSetToList(tset->second);
+	for (auto tset : sets_by_contents) {
+		if (tset.second->type & ST_USED) {
+			addSetToList(tset.second);
 		}
 	}
 
-	for (BOOST_AUTO(iter_tags, single_tags.begin()) ; iter_tags != single_tags.end() ; ++iter_tags) {
-		Tag *tag = iter_tags->second;
+	for (auto iter_tags : single_tags) {
+		Tag *tag = iter_tags.second;
 		if (tag->tag[0] == mapping_prefix) {
 			tag->type |= T_MAPPING;
 		}
@@ -766,70 +775,70 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 	}
 
 	if (!is_binary) {
-		boost_foreach (Set *set, sets_list) {
+		for (auto set : sets_list) {
 			set->reindex(*this);
 		}
-		boost_foreach (Set *set, sets_list) {
+		for (auto set : sets_list) {
 			setAdjustSets(set);
 		}
 	}
-	boost_foreach (Set *set, sets_list) {
+	for (auto set : sets_list) {
 		indexSets(set->number, set);
 	}
 
 	uint32SortedVector sects;
 
-	foreach (RuleVector, rule_by_number, iter_rule, iter_rule_end) {
-		if ((*iter_rule)->section == -1) {
-			before_sections.push_back(*iter_rule);
+	for (auto rule : rule_by_number) {
+		if (rule->section == -1) {
+			before_sections.push_back(rule);
 		}
-		else if ((*iter_rule)->section == -2) {
-			after_sections.push_back(*iter_rule);
+		else if (rule->section == -2) {
+			after_sections.push_back(rule);
 		}
-		else if ((*iter_rule)->section == -3) {
-			null_section.push_back(*iter_rule);
+		else if (rule->section == -3) {
+			null_section.push_back(rule);
 		}
 		else {
-			sects.insert((*iter_rule)->section);
-			rules.push_back(*iter_rule);
+			sects.insert(rule->section);
+			rules.push_back(rule);
 		}
-		if ((*iter_rule)->target) {
+		if (rule->target) {
 			Set *set = 0;
 			if (is_binary) {
-				set = sets_list[(*iter_rule)->target];
+				set = sets_list[rule->target];
 			}
 			else {
-				set = sets_by_contents.find((*iter_rule)->target)->second;
-				(*iter_rule)->target = set->number;
+				set = sets_by_contents.find(rule->target)->second;
+				rule->target = set->number;
 			}
-			indexSetToRule((*iter_rule)->number, set);
-			rules_by_set[(*iter_rule)->target].insert((*iter_rule)->number);
+			indexSetToRule(rule->number, set);
+			rules_by_set[rule->target].insert(rule->number);
 		}
 		else {
-			u_fprintf(ux_stderr, "Warning: Rule on line %u had no target.\n", (*iter_rule)->line);
+			u_fprintf(ux_stderr, "Warning: Rule on line %u had no target.\n", rule->line);
 			u_fflush(ux_stderr);
 		}
-		if (((*iter_rule)->maplist && ((*iter_rule)->maplist->type & ST_CHILD_UNIFY)) || ((*iter_rule)->sublist && ((*iter_rule)->sublist->type & ST_CHILD_UNIFY))) {
-			(*iter_rule)->flags |= FL_CAPTURE_UNIF;
+		if ((rule->maplist && (rule->maplist->type & ST_CHILD_UNIFY)) || (rule->sublist && (rule->sublist->type & ST_CHILD_UNIFY))) {
+			rule->flags |= FL_CAPTURE_UNIF;
 		}
 		if (is_binary) {
 			continue;
 		}
-		if ((*iter_rule)->childset1) {
-			Set *set = sets_by_contents.find((*iter_rule)->childset1)->second;
-			(*iter_rule)->childset1 = set->number;
+		if (rule->childset1) {
+			Set *set = sets_by_contents.find(rule->childset1)->second;
+			rule->childset1 = set->number;
 		}
-		if ((*iter_rule)->childset2) {
-			Set *set = sets_by_contents.find((*iter_rule)->childset2)->second;
-			(*iter_rule)->childset2 = set->number;
+		if (rule->childset2) {
+			Set *set = sets_by_contents.find(rule->childset2)->second;
+			rule->childset2 = set->number;
 		}
-		if ((*iter_rule)->dep_target) {
-			contextAdjustTarget((*iter_rule)->dep_target);
+		if (rule->dep_target) {
+			contextAdjustTarget(rule->dep_target);
 		}
-		boost_foreach (ContextualTest *test, (*iter_rule)->tests) {
+		for (auto test : rule->tests) {
 			contextAdjustTarget(test);
 		}
-		boost_foreach (ContextualTest *test, (*iter_rule)->dep_tests) {
+		for (auto test : rule->dep_tests) {
 			contextAdjustTarget(test);
 		}
 	}
@@ -849,7 +858,7 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 
 	sets_by_contents.clear();
 
-	boost_foreach (Set *to, sets_list) {
+	for (auto to : sets_list) {
 		if (to->type & ST_STATIC) {
 			uint32_t nhash = hash_value(to->name);
 			const uint32_t cnum = to->number;
@@ -864,14 +873,14 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 					CG3Quit(1);
 				}
 				else {
-					for (uint32_t seed=0 ; seed<1000 ; ++seed) {
-						if (sets_by_name.find(nhash+seed) == sets_by_name.end()) {
+					for (uint32_t seed = 0; seed < 1000; ++seed) {
+						if (sets_by_name.find(nhash + seed) == sets_by_name.end()) {
 							if (verbosity_level > 0) {
 								u_fprintf(ux_stderr, "Warning: Static set %S got hash seed %u.\n", to->name.c_str(), seed);
 								u_fflush(ux_stderr);
 							}
 							set_name_seeds[to->name] = seed;
-							sets_by_name[nhash+seed] = cnum;
+							sets_by_name[nhash + seed] = cnum;
 							break;
 						}
 					}
@@ -880,8 +889,122 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 		}
 	}
 
+	// Gather knowledge of which sets have varstrings
+	boost::dynamic_bitset<> sets_vstr(sets_list.size());
+	bool did = true;
+	while (did) {
+		did = false;
+		for (auto set : sets_list) {
+			if (sets_vstr.test(set->number)) {
+				continue;
+			}
+			for (auto iset : set->sets) {
+				if (sets_vstr.test(iset)) {
+					sets_vstr.set(set->number);
+					did = true;
+					break;
+				}
+			}
+			if (trie_hasType(set->trie, T_VARSTRING) || trie_hasType(set->trie_special, T_VARSTRING)) {
+				sets_vstr.set(set->number);
+				did = true;
+			}
+		}
+	}
+
+	// Gather knowledge of which contexts use unification or varstrings
+	bc::flat_set<ContextualTest*> nk;
+	did = true;
+	while (did) {
+		did = false;
+
+		for (auto cntx : contexts) {
+			ContextualTest *t = cntx.second;
+
+			if (nk.count(t)) {
+				continue;
+			}
+
+			if (t->tmpl && nk.count(t->tmpl)) {
+				//u_fprintf(ux_stderr, "added tmpl %u\n", t->line);
+				did |= nk.insert(t).second;
+				continue;
+			}
+			if (t->linked && nk.count(t->linked)) {
+				//u_fprintf(ux_stderr, "added linked %u\n", t->line);
+				did |= nk.insert(t).second;
+				continue;
+			}
+			if (t->target && (sets_list[t->target]->type & MASK_ST_UNIFY)) {
+				//u_fprintf(ux_stderr, "added target %u\n", t->line);
+				did |= nk.insert(t).second;
+				continue;
+			}
+			if (t->target && sets_vstr.test(t->target)) {
+				//u_fprintf(ux_stderr, "added target %u\n", t->line);
+				did |= nk.insert(t).second;
+				continue;
+			}
+			if (t->barrier && (sets_list[t->barrier]->type & MASK_ST_UNIFY)) {
+				//u_fprintf(ux_stderr, "added barrier %u\n", t->line);
+				did |= nk.insert(t).second;
+				continue;
+			}
+			if (t->barrier && sets_vstr.test(t->barrier)) {
+				//u_fprintf(ux_stderr, "added target %u\n", t->line);
+				did |= nk.insert(t).second;
+				continue;
+			}
+			if (t->cbarrier && (sets_list[t->cbarrier]->type & MASK_ST_UNIFY)) {
+				//u_fprintf(ux_stderr, "added cbarrier %u\n", t->line);
+				did |= nk.insert(t).second;
+				continue;
+			}
+			if (t->cbarrier && sets_vstr.test(t->cbarrier)) {
+				//u_fprintf(ux_stderr, "added target %u\n", t->line);
+				did |= nk.insert(t).second;
+				continue;
+			}
+		}
+	}
+
+	for (auto r : rule_by_number) {
+		// Determine whether this rule probably needs KEEPORDER
+		if (r->flags & RF_KEEPORDER) {
+			continue;
+		}
+		/* While this is a good indication that the unified set is used in the target, it is not 100%
+		if (r->target && (sets_list[r->target]->type & MASK_ST_UNIFY)) {
+			continue;
+		}
+		//*/
+		bool needs = false;
+		if (r->dep_target && nk.count(r->dep_target)) {
+			needs = true;
+		}
+		for (auto cntx : r->tests) {
+			if (nk.count(cntx)) {
+				needs = true;
+			}
+		}
+		for (auto cntx : r->dep_tests) {
+			if (nk.count(cntx)) {
+				needs = true;
+			}
+		}
+		if (needs) {
+			r->flags |= RF_KEEPORDER;
+			/* We can do the whole thing fully automatically, so explicit KEEPORDER will eventually be deprecated
+			if (verbosity_level) {
+				u_fprintf(ux_stderr, "Warning: Rule on line %u probably needs KEEPORDER.\n", r->line);
+				u_fflush(ux_stderr);
+			}
+			//*/
+		}
+	}
+
 	if (used_tags) {
-		for (BOOST_AUTO(iter_tags, single_tags.begin()); iter_tags != single_tags.end(); ++iter_tags) {
+		for (auto iter_tags = single_tags.begin(); iter_tags != single_tags.end(); ++iter_tags) {
 			Tag *tag = iter_tags->second;
 			if (tag->type & T_USED) {
 				UString tmp(tag->toUString(true));
@@ -893,7 +1016,7 @@ void Grammar::reindex(bool unused_sets, bool used_tags) {
 }
 
 inline void trie_indexToRule(const trie_t& trie, Grammar& grammar, uint32_t r) {
-	boost_foreach (const trie_t::value_type& kv, trie) {
+	for (auto& kv : trie) {
 		grammar.indexTagToRule(kv.first->hash, r);
 		if (kv.second.trie) {
 			trie_indexToRule(*kv.second.trie, grammar, r);
@@ -902,7 +1025,7 @@ inline void trie_indexToRule(const trie_t& trie, Grammar& grammar, uint32_t r) {
 }
 
 void Grammar::indexSetToRule(uint32_t r, Set *s) {
-	if (s->type & (ST_SPECIAL|ST_TAG_UNIFY)) {
+	if (s->type & (ST_SPECIAL | ST_TAG_UNIFY)) {
 		indexTagToRule(tag_any, r);
 		return;
 	}
@@ -910,7 +1033,7 @@ void Grammar::indexSetToRule(uint32_t r, Set *s) {
 	trie_indexToRule(s->trie, *this, r);
 	trie_indexToRule(s->trie_special, *this, r);
 
-	for (uint32_t i=0 ; i<s->sets.size() ; ++i) {
+	for (uint32_t i = 0; i < s->sets.size(); ++i) {
 		Set *set = sets_list[s->sets[i]];
 		indexSetToRule(r, set);
 	}
@@ -921,7 +1044,7 @@ void Grammar::indexTagToRule(uint32_t t, uint32_t r) {
 }
 
 inline void trie_indexToSet(const trie_t& trie, Grammar& grammar, uint32_t r) {
-	boost_foreach (const trie_t::value_type& kv, trie) {
+	for (auto& kv : trie) {
 		grammar.indexTagToSet(kv.first->hash, r);
 		if (kv.second.trie) {
 			trie_indexToSet(*kv.second.trie, grammar, r);
@@ -930,7 +1053,7 @@ inline void trie_indexToSet(const trie_t& trie, Grammar& grammar, uint32_t r) {
 }
 
 void Grammar::indexSets(uint32_t r, Set *s) {
-	if (s->type & (ST_SPECIAL|ST_TAG_UNIFY)) {
+	if (s->type & (ST_SPECIAL | ST_TAG_UNIFY)) {
 		indexTagToSet(tag_any, r);
 		return;
 	}
@@ -938,7 +1061,7 @@ void Grammar::indexSets(uint32_t r, Set *s) {
 	trie_indexToSet(s->trie, *this, r);
 	trie_indexToSet(s->trie_special, *this, r);
 
-	for (uint32_t i=0 ; i<s->sets.size() ; ++i) {
+	for (uint32_t i = 0; i < s->sets.size(); ++i) {
 		Set *set = sets_list[s->sets[i]];
 		indexSets(r, set);
 	}
@@ -957,7 +1080,7 @@ void Grammar::setAdjustSets(Set *s) {
 	}
 	s->type &= ~ST_USED;
 
-	for (uint32_t i = 0; i<s->sets.size(); ++i) {
+	for (uint32_t i = 0; i < s->sets.size(); ++i) {
 		Set *set = sets_by_contents.find(s->sets[i])->second;
 		s->sets[i] = set->number;
 		setAdjustSets(set);
@@ -983,7 +1106,7 @@ void Grammar::contextAdjustTarget(ContextualTest *test) {
 		test->cbarrier = set->number;
 	}
 
-	boost_foreach (ContextualTest *tor, test->ors) {
+	for (auto tor : test->ors) {
 		contextAdjustTarget(tor);
 	}
 	if (test->tmpl) {
@@ -993,5 +1116,4 @@ void Grammar::contextAdjustTarget(ContextualTest *test) {
 		contextAdjustTarget(test->linked);
 	}
 }
-
 }
