@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2016, GrammarSoft ApS
+* Copyright (C) 2007-2017, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -47,7 +47,7 @@ inline bool TagSet_SubsetOf_TSet(const TagSortedVector& a, const T& b) {
 	}
 	//*/
 	typename T::const_iterator bi = b.lower_bound((*a.begin())->hash);
-	boost_foreach (Tag *ai, a) {
+	for (auto ai : a) {
 		while (bi != b.end() && *bi < ai->hash) {
 			++bi;
 		}
@@ -76,7 +76,7 @@ uint32_t GrammarApplicator::doesTagMatchRegexp(uint32_t test, const Tag& tag, bo
 	else {
 		const Tag& itag = *(single_tags.find(test)->second);
 		UErrorCode status = U_ZERO_ERROR;
-		uregex_setText(tag.regexp, itag.tag.c_str(), itag.tag.length(), &status);
+		uregex_setText(tag.regexp, itag.tag.c_str(), itag.tag.size(), &status);
 		if (status != U_ZERO_ERROR) {
 			u_fprintf(ux_stderr, "Error: uregex_setText(MatchSet) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
 			CG3Quit(1);
@@ -91,7 +91,8 @@ uint32_t GrammarApplicator::doesTagMatchRegexp(uint32_t test, const Tag& tag, bo
 		}
 		if (match) {
 			int32_t gc = uregex_groupCount(tag.regexp, &status);
-			if (gc > 0) {
+			// ToDo: Allow regex captures from dependency target contexts without any captures in normal target contexts
+			if (gc > 0 && regexgrps.second != 0) {
 				UChar tmp[1024];
 				for (int i = 1; i <= gc; ++i) {
 					tmp[0] = 0;
@@ -126,7 +127,7 @@ uint32_t GrammarApplicator::doesTagMatchIcase(uint32_t test, const Tag& tag, boo
 	else {
 		const Tag& itag = *(single_tags.find(test)->second);
 		UErrorCode status = U_ZERO_ERROR;
-		if (u_strCaseCompare(tag.tag.c_str(), tag.tag.length(), itag.tag.c_str(), itag.tag.length(), U_FOLD_CASE_DEFAULT, &status) == 0) {
+		if (u_strCaseCompare(tag.tag.c_str(), tag.tag.size(), itag.tag.c_str(), itag.tag.size(), U_FOLD_CASE_DEFAULT, &status) == 0) {
 			match = itag.hash;
 		}
 		if (status != U_ZERO_ERROR) {
@@ -143,6 +144,57 @@ uint32_t GrammarApplicator::doesTagMatchIcase(uint32_t test, const Tag& tag, boo
 	return match;
 }
 
+// ToDo: Remove for real ordered mode
+uint32_t GrammarApplicator::doesRegexpMatchLine(const Reading& reading, const Tag& tag, bool bypass_index) {
+	uint32_t match = 0;
+	uint32_t ih = reading.tags_string_hash;
+	if (!bypass_index && index_matches(index_regexp_no, ih)) {
+		match = 0;
+	}
+	else if (!bypass_index && index_matches(index_regexp_yes, ih)) {
+		match = reading.tags_string_hash;
+	}
+	else {
+		UErrorCode status = U_ZERO_ERROR;
+		uregex_setText(tag.regexp, reading.tags_string.c_str(), reading.tags_string.size(), &status);
+		if (status != U_ZERO_ERROR) {
+			u_fprintf(ux_stderr, "Error: uregex_setText(MatchSet) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
+			CG3Quit(1);
+		}
+		status = U_ZERO_ERROR;
+		if (uregex_find(tag.regexp, -1, &status)) {
+			match = reading.tags_string_hash;
+		}
+		if (status != U_ZERO_ERROR) {
+			u_fprintf(ux_stderr, "Error: uregex_find(MatchSet) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
+			CG3Quit(1);
+		}
+		if (match) {
+			int32_t gc = uregex_groupCount(tag.regexp, &status);
+			// ToDo: Allow regex captures from dependency target contexts without any captures in normal target contexts
+			if (gc > 0 && regexgrps.second != 0) {
+				UChar tmp[1024];
+				for (int i = 1; i <= gc; ++i) {
+					tmp[0] = 0;
+					int32_t len = uregex_group(tag.regexp, i, tmp, 1024, &status);
+					regexgrps.second->resize(std::max(static_cast<size_t>(regexgrps.first) + 1, regexgrps.second->size()));
+					UnicodeString& ucstr = (*regexgrps.second)[regexgrps.first];
+					ucstr.remove();
+					ucstr.append(tmp, len);
+					++regexgrps.first;
+				}
+			}
+			else {
+				index_regexp_yes.insert(ih);
+			}
+		}
+		else {
+			index_regexp_no.insert(ih);
+		}
+	}
+	return match;
+}
+
 /**
  * Tests whether a given reading matches a given tag's stored regular expression.
  *
@@ -152,9 +204,14 @@ uint32_t GrammarApplicator::doesTagMatchIcase(uint32_t test, const Tag& tag, boo
 uint32_t GrammarApplicator::doesRegexpMatchReading(const Reading& reading, const Tag& tag, bool bypass_index) {
 	uint32_t match = 0;
 
+	// ToDo: Remove for real ordered mode
+	if (tag.type & T_REGEXP_LINE) {
+		return doesRegexpMatchLine(reading, tag, bypass_index);
+	}
+
 	// Grammar::reindex() will do a one-time pass to mark any potential matching tag as T_TEXTUAL
-	foreach (mter, reading.tags_textual) {
-		match = doesTagMatchRegexp(*mter, tag, bypass_index);
+	for (auto mter : reading.tags_textual) {
+		match = doesTagMatchRegexp(mter, tag, bypass_index);
 		if (match) {
 			break;
 		}
@@ -205,8 +262,8 @@ uint32_t GrammarApplicator::doesTagMatchReading(const Reading& reading, const Ta
 		match = doesRegexpMatchReading(reading, tag, bypass_index);
 	}
 	else if (tag.type & T_CASE_INSENSITIVE) {
-		foreach (mter, reading.tags_textual) {
-			match = doesTagMatchIcase(*mter, tag, bypass_index);
+		for (auto mter : reading.tags_textual) {
+			match = doesTagMatchIcase(mter, tag, bypass_index);
 			if (match) {
 				break;
 			}
@@ -240,18 +297,18 @@ uint32_t GrammarApplicator::doesTagMatchReading(const Reading& reading, const Ta
 			}
 		}
 		else {
-			foreach (mter, reading.tags_textual) {
-				const Tag& itag = *(single_tags.find(*mter)->second);
+			for (auto mter : reading.tags_textual) {
+				const Tag& itag = *(single_tags.find(mter)->second);
 				if (!(itag.type & (T_BASEFORM | T_WORDFORM))) {
 					match = itag.hash;
 					if (unif_mode) {
 						if (unif_last_textual) {
-							if (unif_last_textual != *mter) {
+							if (unif_last_textual != mter) {
 								match = 0;
 							}
 						}
 						else {
-							unif_last_textual = *mter;
+							unif_last_textual = mter;
 						}
 					}
 				}
@@ -262,13 +319,13 @@ uint32_t GrammarApplicator::doesTagMatchReading(const Reading& reading, const Ta
 		}
 	}
 	else if (tag.type & T_NUMERICAL) {
-		boost_foreach (const Reading::tags_numerical_t::value_type& mter, reading.tags_numerical) {
+		for (auto mter : reading.tags_numerical) {
 			const Tag& itag = *(mter.second);
-			int32_t compval = tag.comparison_val;
-			if (compval == INT_MIN) {
+			double compval = tag.comparison_val;
+			if (compval <= NUMERIC_MIN) {
 				compval = reading.parent->getMin(tag.comparison_hash);
 			}
-			else if (compval == INT_MAX) {
+			else if (compval >= NUMERIC_MAX) {
 				compval = reading.parent->getMax(tag.comparison_hash);
 			}
 			if (tag.comparison_hash == itag.comparison_hash) {
@@ -441,7 +498,7 @@ uint32_t GrammarApplicator::doesTagMatchReading(const Reading& reading, const Ta
 }
 
 bool GrammarApplicator::doesSetMatchReading_trie(const Reading& reading, const Set& theset, const trie_t& trie, bool unif_mode) {
-	boost_foreach (const trie_t::value_type& kv, trie) {
+	for (auto& kv : trie) {
 		bool match = (doesTagMatchReading(reading, *kv.first, unif_mode) != 0);
 		if (match) {
 			if (kv.first->type & T_FAILFAST) {
@@ -449,7 +506,7 @@ bool GrammarApplicator::doesSetMatchReading_trie(const Reading& reading, const S
 			}
 			if (kv.second.terminal) {
 				if (unif_mode) {
-					BOOST_AUTO(it, unif_tags->find(theset.number));
+					auto it = unif_tags->find(theset.number);
 					if (it != unif_tags->end() && it->second != &kv) {
 						continue;
 					}
@@ -478,7 +535,7 @@ bool GrammarApplicator::doesSetMatchReading_tags(const Reading& reading, const S
 	bool retval = false;
 
 	if (!theset.ff_tags.empty()) {
-		boost_foreach (const Tag *tag, theset.ff_tags) {
+		for (auto tag : theset.ff_tags) {
 			if (doesTagMatchReading(reading, *tag, unif_mode)) {
 				return false;
 			}
@@ -494,7 +551,7 @@ bool GrammarApplicator::doesSetMatchReading_tags(const Reading& reading, const S
 			if (*oiter == iiter->first->hash) {
 				if (iiter->second.terminal) {
 					if (unif_mode) {
-						BOOST_AUTO(it, unif_tags->find(theset.number));
+						auto it = unif_tags->find(theset.number);
 						if (it != unif_tags->end() && it->second != &*iiter) {
 							++iiter;
 							continue;
@@ -587,10 +644,10 @@ bool GrammarApplicator::doesSetMatchReading(const Reading& reading, const uint32
 		}
 		// Subsequent times, test whether any of the previously stored sets match the reading
 		else {
-			BOOST_AUTO(sets, ss_u32sv.get());
-			foreach (usi, *unif_sets) {
-				if (doesSetMatchReading(reading, *usi, bypass_index, unif_mode)) {
-					sets->insert(*usi);
+			auto sets = ss_u32sv.get();
+			for (auto usi : *unif_sets) {
+				if (doesSetMatchReading(reading, usi, bypass_index, unif_mode)) {
+					sets->insert(usi);
 				}
 			}
 			retval = !sets->empty();
@@ -653,7 +710,7 @@ bool GrammarApplicator::doesSetMatchReading(const Reading& reading, const uint32
 		if (unif_mode || (theset.type & ST_TAG_UNIFY)) {
 			const void *tag = 0;
 			for (size_t i = 0; i < size; ++i) {
-				BOOST_AUTO(it, unif_tags->find(theset.sets[i]));
+				auto it = unif_tags->find(theset.sets[i]);
 				if (it != unif_tags->end()) {
 					tag = it->second;
 					break;
@@ -702,15 +759,20 @@ inline bool _check_options(std::vector<Reading*>& rv, uint32_t options, size_t n
 
 inline bool GrammarApplicator::doesSetMatchCohort_testLinked(Cohort& cohort, const Set& theset, dSMC_Context *context) {
 	bool retval = true;
+	bool reset = false;
 	const ContextualTest *linked = 0;
-	inc_dec<size_t> ic;
+	Cohort *min = 0;
+	Cohort *max = 0;
 
 	if (context->test && context->test->linked) {
 		linked = context->test->linked;
 	}
-	else if (!tmpl_cntxs.empty() && tmpl_cntx_pos < tmpl_cntxs.size()) {
-		ic.inc(tmpl_cntx_pos);
-		linked = tmpl_cntxs[tmpl_cntxs.size() - tmpl_cntx_pos].test;
+	else if (!tmpl_cntx.linked.empty()) {
+		min = tmpl_cntx.min;
+		max = tmpl_cntx.max;
+		linked = tmpl_cntx.linked.back();
+		tmpl_cntx.linked.pop_back();
+		reset = true;
 	}
 	if (linked) {
 		if (!context->did_test) {
@@ -726,13 +788,20 @@ inline bool GrammarApplicator::doesSetMatchCohort_testLinked(Cohort& cohort, con
 		}
 		retval = context->matched_tests;
 	}
+	if (reset) {
+		tmpl_cntx.linked.push_back(linked);
+	}
+	if (!retval) {
+		tmpl_cntx.min = min;
+		tmpl_cntx.max = max;
+	}
 	return retval;
 }
 
 inline bool GrammarApplicator::doesSetMatchCohort_helper(Cohort& cohort, Reading& reading, const Set& theset, dSMC_Context *context) {
 	bool retval = false;
-	BOOST_AUTO(utags, ss_utags.get());
-	BOOST_AUTO(usets, ss_u32sv.get());
+	auto utags = ss_utags.get();
+	auto usets = ss_u32sv.get();
 	uint8_t orz = regexgrps.first;
 
 	if (context && !(current_rule->flags & FL_CAPTURE_UNIF) && (theset.type & ST_CHILD_UNIFY)) {
@@ -799,8 +868,7 @@ bool GrammarApplicator::doesSetMatchCohortNormal(Cohort& cohort, const uint32_t
 		if (lists[i] == 0) {
 			continue;
 		}
-		foreach (iter, *lists[i]) {
-			Reading *reading = *iter;
+		for (auto reading : *lists[i]) {
 			if (context && context->test) {
 				// ToDo: Barriers need some way to escape sub-readings
 				reading = get_sub_reading(reading, context->test->offset_sub);
@@ -853,8 +921,7 @@ bool GrammarApplicator::doesSetMatchCohortCareful(Cohort& cohort, const uint32_t
 		if (lists[i] == 0) {
 			continue;
 		}
-		foreach (iter, *lists[i]) {
-			Reading *reading = *iter;
+		for (auto reading : *lists[i]) {
 			if (context && context->test) {
 				// ToDo: Barriers need some way to escape sub-readings
 				reading = get_sub_reading(reading, context->test->offset_sub);
