@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2016, GrammarSoft ApS
+* Copyright (C) 2007-2017, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -102,7 +102,7 @@ cg3_grammar *cg3_grammar_load(const char *filename) {
 	grammar->ux_stderr = ux_stderr;
 	grammar->ux_stdout = ux_stdout;
 
-	boost::scoped_ptr<IGrammarParser> parser;
+	std::unique_ptr<IGrammarParser> parser;
 
 	if (cbuffers[0][0] == 'C' && cbuffers[0][1] == 'G' && cbuffers[0][2] == '3' && cbuffers[0][3] == 'B') {
 		u_fprintf(ux_stderr, "CG3 Info: Binary grammar detected.\n");
@@ -202,7 +202,7 @@ inline Reading *_reading_copy(Cohort *nc, Reading *oldr, bool is_sub = false) {
 	insert_if_exists(nr->parent->possible_sets, ga->grammar->sets_any);
 	ga->addTagToReading(*nr, nc->wordform);
 	TagList mappings;
-	boost_foreach (uint32_t tag, oldr->tags_list) {
+	for (auto tag : oldr->tags_list) {
 		Tag *nt = _tag_copy(oldr->parent->parent->parent->parent, nc->parent->parent->parent, tag);
 		if (nt->type & T_MAPPING || nt->tag[0] == ga->grammar->mapping_prefix) {
 			mappings.push_back(nt);
@@ -223,7 +223,7 @@ inline Reading *_reading_copy(Cohort *nc, Reading *oldr, bool is_sub = false) {
 inline Cohort *_cohort_copy(SingleWindow *ns, Cohort *oc) {
 	Cohort *nc = alloc_cohort(ns);
 	nc->wordform = _tag_copy(ns->parent->parent, oc->wordform);
-	boost_foreach (Reading *r, oc->readings) {
+	for (auto r : oc->readings) {
 		Reading *nr = _reading_copy(nc, r);
 		nc->appendReading(nr);
 	}
@@ -241,7 +241,7 @@ cg3_sentence *cg3_sentence_copy(cg3_sentence *sentence_, cg3_applicator *applica
 	applicator->initEmptySingleWindow(current);
 	current->has_enclosures = sentence->has_enclosures;
 	current->text = sentence->text;
-	boost_foreach (Cohort *c, sentence->cohorts) {
+	for (auto c : sentence->cohorts) {
 		Cohort *nc = _cohort_copy(current, c);
 		current->appendCohort(nc);
 	}
@@ -323,10 +323,10 @@ void cg3_cohort_getrelation_u(cg3_cohort *cohort_, const UChar *rel, uint32_t *r
 	GrammarApplicator *ga = cohort->parent->parent->parent;
 
 	if ((cohort->type & CT_RELATED) && !cohort->relations.empty()) {
-		foreach (miter, cohort->relations) {
-			foreach (siter, miter->second) {
-				if (u_strcmp(ga->single_tags.find(miter->first)->second->tag.c_str(), rel) == 0) {
-					*rel_parent = *siter;
+		for (auto miter : cohort->relations) {
+			for (auto siter : miter->second) {
+				if (u_strcmp(ga->single_tags.find(miter.first)->second->tag.c_str(), rel) == 0) {
+					*rel_parent = siter;
 				}
 			}
 		}
@@ -509,7 +509,7 @@ const char *cg3_tag_gettext_u8(cg3_tag *tag_) {
 	Tag *tag = static_cast<Tag*>(tag_);
 	UErrorCode status = U_ZERO_ERROR;
 
-	u_strToUTF8(&cbuffers[0][0], CG3_BUFFER_SIZE - 1, 0, tag->tag.c_str(), tag->tag.length(), &status);
+	u_strToUTF8(&cbuffers[0][0], CG3_BUFFER_SIZE - 1, 0, tag->tag.c_str(), tag->tag.size(), &status);
 	if (U_FAILURE(status)) {
 		u_fprintf(ux_stderr, "CG3 Error: Failed to convert text from UChar to UTF-8. Status = %s\n", u_errorName(status));
 		return 0;
@@ -529,7 +529,7 @@ const uint32_t *cg3_tag_gettext_u32(cg3_tag *tag_) {
 
 	UChar32 *tmp = reinterpret_cast<UChar32*>(&cbuffers[0][0]);
 
-	u_strToUTF32(tmp, (CG3_BUFFER_SIZE / sizeof(UChar32)) - 1, 0, tag->tag.c_str(), tag->tag.length(), &status);
+	u_strToUTF32(tmp, (CG3_BUFFER_SIZE / sizeof(UChar32)) - 1, 0, tag->tag.c_str(), tag->tag.size(), &status);
 	if (U_FAILURE(status)) {
 		u_fprintf(ux_stderr, "CG3 Error: Failed to convert text from UChar to UTF-32. Status = %s\n", u_errorName(status));
 		return 0;
@@ -544,7 +544,7 @@ const wchar_t *cg3_tag_gettext_w(cg3_tag *tag_) {
 
 	wchar_t *tmp = reinterpret_cast<wchar_t*>(&cbuffers[0][0]);
 
-	u_strToWCS(tmp, (CG3_BUFFER_SIZE / sizeof(wchar_t)) - 1, 0, tag->tag.c_str(), tag->tag.length(), &status);
+	u_strToWCS(tmp, (CG3_BUFFER_SIZE / sizeof(wchar_t)) - 1, 0, tag->tag.c_str(), tag->tag.size(), &status);
 	if (U_FAILURE(status)) {
 		u_fprintf(ux_stderr, "CG3 Error: Failed to convert text from UChar to UTF-32. Status = %s\n", u_errorName(status));
 		return 0;
@@ -552,3 +552,23 @@ const wchar_t *cg3_tag_gettext_w(cg3_tag *tag_) {
 
 	return tmp;
 }
+
+// These 3 from Paul Meurer <paul.meurer@uni.no>
+size_t cg3_cohort_numdelreadings(cg3_cohort *cohort_) {
+	Cohort *cohort = static_cast<Cohort*>(cohort_);
+	return cohort->deleted.size();
+}
+
+cg3_reading *cg3_cohort_getdelreading(cg3_cohort *cohort_, size_t which) {
+	Cohort *cohort = static_cast<Cohort*>(cohort_);
+	ReadingList::iterator it = cohort->deleted.begin();
+	std::advance(it, which);
+	return *it;
+}
+
+size_t cg3_reading_gettrace_ruletype(cg3_reading *reading_, size_t which) {
+	Reading *reading = static_cast<Reading*>(reading_);
+	Grammar *grammar = reading->parent->parent->parent->parent->grammar;
+	const Rule *r = grammar->rule_by_number[reading->hit_by[which]];
+	return r->type;
+}
