@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2015, GrammarSoft ApS
+* Copyright (C) 2007-2017, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -32,7 +32,7 @@ using namespace Options;
 using CG3::CG3Quit;
 void GAppSetOpts(CG3::GrammarApplicator& applicator, UConverter *conv);
 
-int main(int argc, char* argv[]) {
+int main(int argc, char *argv[]) {
 	UFILE *ux_stdin = 0;
 	UFILE *ux_stdout = 0;
 	UFILE *ux_stderr = 0;
@@ -78,13 +78,13 @@ int main(int argc, char* argv[]) {
 		fprintf(out, "Options:\n");
 
 		size_t longest = 0;
-		for (uint32_t i=0 ; i<NUM_OPTIONS ; i++) {
+		for (uint32_t i = 0; i < NUM_OPTIONS; i++) {
 			if (options[i].description) {
 				size_t len = strlen(options[i].longName);
 				longest = std::max(longest, len);
 			}
 		}
-		for (uint32_t i=0 ; i<NUM_OPTIONS ; i++) {
+		for (uint32_t i = 0; i < NUM_OPTIONS; i++) {
 			if (options[i].description) {
 				fprintf(out, " ");
 				if (options[i].shortName) {
@@ -109,7 +109,7 @@ int main(int argc, char* argv[]) {
 	fflush(out);
 	fflush(stderr);
 
-	if (options[SHOW_UNUSED_SETS].doesOccur || options[SHOW_SET_HASHES].doesOccur) {
+	if (options[SHOW_UNUSED_SETS].doesOccur || options[SHOW_SET_HASHES].doesOccur || options[DUMP_AST].doesOccur) {
 		options[GRAMMAR_ONLY].doesOccur = true;
 	}
 
@@ -138,8 +138,8 @@ int main(int argc, char* argv[]) {
 	ucnv_setDefaultName("UTF-8");
 	const char *codepage_default = ucnv_getDefaultName();
 	const char *codepage_grammar = codepage_default;
-	const char *codepage_input   = codepage_grammar;
-	const char *codepage_output  = codepage_grammar;
+	const char *codepage_input = codepage_grammar;
+	const char *codepage_output = codepage_grammar;
 
 	if (options[CODEPAGE_GRAMMAR].doesOccur) {
 		codepage_grammar = options[CODEPAGE_GRAMMAR].value;
@@ -235,10 +235,14 @@ int main(int argc, char* argv[]) {
 		if (options[VERBOSE].doesOccur) {
 			std::cerr << "Info: Binary grammar detected." << std::endl;
 		}
+		if (options[DUMP_AST].doesOccur) {
+			std::cerr << "Error: --dump-ast is for textual grammars only!" << std::endl;
+			CG3Quit(1);
+		}
 		parser = new CG3::BinaryGrammar(grammar, ux_stderr);
 	}
 	else {
-		parser = new CG3::TextualParser(grammar, ux_stderr);
+		parser = new CG3::TextualParser(grammar, ux_stderr, options[DUMP_AST].doesOccur != 0);
 	}
 	if (options[VERBOSE].doesOccur) {
 		if (options[VERBOSE].value) {
@@ -256,7 +260,7 @@ int main(int argc, char* argv[]) {
 	parser->setCompatible(options[VISLCGCOMPAT].doesOccur != 0);
 
 	if (options[VERBOSE].doesOccur) {
-		std::cerr << "Initialization took " << (clock()-main_timer)/(double)CLOCKS_PER_SEC << " seconds." << std::endl;
+		std::cerr << "Initialization took " << (clock() - main_timer) / (double)CLOCKS_PER_SEC << " seconds." << std::endl;
 	}
 	main_timer = clock();
 
@@ -265,10 +269,14 @@ int main(int argc, char* argv[]) {
 		CG3Quit(1);
 	}
 
+	if (options[DUMP_AST].doesOccur) {
+		dynamic_cast<CG3::TextualParser*>(parser)->print_ast(ux_stdout);
+	}
+
 	if (options[MAPPING_PREFIX].doesOccur) {
 		UConverter *conv = ucnv_open(codepage_cli, &status);
 		size_t sn = strlen(options[MAPPING_PREFIX].value);
-		CG3::UString buf(sn*3, 0);
+		CG3::UString buf(sn * 3, 0);
 		ucnv_toUChars(conv, &buf[0], buf.size(), options[MAPPING_PREFIX].value, sn, &status);
 		if (grammar.is_binary && grammar.mapping_prefix != buf[0]) {
 			std::cerr << "Error: Mapping prefix must match the one used for compiling the binary grammar!" << std::endl;
@@ -286,7 +294,7 @@ int main(int argc, char* argv[]) {
 	parser = 0;
 
 	if (options[VERBOSE].doesOccur) {
-		std::cerr << "Parsing grammar took " << (clock()-main_timer)/(double)CLOCKS_PER_SEC << " seconds." << std::endl;
+		std::cerr << "Parsing grammar took " << (clock() - main_timer) / (double)CLOCKS_PER_SEC << " seconds." << std::endl;
 	}
 	main_timer = clock();
 
@@ -303,21 +311,6 @@ int main(int argc, char* argv[]) {
 		}
 	}
 
-	if (grammar.is_binary) {
-		if (options[GRAMMAR_BIN].doesOccur || options[GRAMMAR_OUT].doesOccur) {
-			std::cerr << "Error: Binary grammars cannot be rewritten." << std::endl;
-			CG3Quit(1);
-		}
-		if (options[STATISTICS].doesOccur) {
-			std::cerr << "Error: Statistics cannot be gathered with a binary grammar." << std::endl;
-			CG3Quit(1);
-		}
-		if (options[OPTIMIZE_UNSAFE].doesOccur || options[OPTIMIZE_SAFE].doesOccur) {
-			std::cerr << "Error: Binary grammars cannot be further optimized." << std::endl;
-			CG3Quit(1);
-		}
-	}
-
 	if (options[STATISTICS].doesOccur && !(options[GRAMMAR_BIN].doesOccur || options[GRAMMAR_OUT].doesOccur)) {
 		std::cerr << "Error: Does not make sense to gather statistics if you are not writing the compiled grammar back out somehow." << std::endl;
 		CG3Quit(1);
@@ -343,21 +336,21 @@ int main(int argc, char* argv[]) {
 		applicator.runGrammarOnText(instream, ux_stdout);
 
 		if (options[VERBOSE].doesOccur) {
-			std::cerr << "Applying grammar on input took " << (clock()-main_timer)/(double)CLOCKS_PER_SEC << " seconds." << std::endl;
+			std::cerr << "Applying grammar on input took " << (clock() - main_timer) / (double)CLOCKS_PER_SEC << " seconds." << std::endl;
 		}
 		main_timer = clock();
 	}
 
 	if (options[OPTIMIZE_UNSAFE].doesOccur) {
 		std::vector<uint32_t> bad;
-		foreach (CG3::RuleVector, grammar.rule_by_number, ir, ir_end) {
-			if ((*ir)->num_match == 0) {
-				bad.push_back((*ir)->number);
+		for (auto ir : grammar.rule_by_number) {
+			if (ir->num_match == 0) {
+				bad.push_back(ir->number);
 			}
 		}
-		reverse_foreach (std::vector<uint32_t>, bad, br, br_end) {
+		reverse_foreach (br, bad) {
 			CG3::Rule *r = grammar.rule_by_number[*br];
-			grammar.rule_by_number.erase(grammar.rule_by_number.begin()+*br);
+			grammar.rule_by_number.erase(grammar.rule_by_number.begin() + *br);
 			grammar.destroyRule(r);
 		}
 		std::cerr << "Optimizer removed " << bad.size() << " rules." << std::endl;
@@ -366,17 +359,17 @@ int main(int argc, char* argv[]) {
 	}
 	if (options[OPTIMIZE_SAFE].doesOccur) {
 		CG3::RuleVector bad;
-		foreach (CG3::RuleVector, grammar.rule_by_number, ir, ir_end) {
-			if ((*ir)->num_match == 0) {
-				bad.push_back(*ir);
+		for (auto ir : grammar.rule_by_number) {
+			if (ir->num_match == 0) {
+				bad.push_back(ir);
 			}
 		}
-		reverse_foreach (CG3::RuleVector, bad, br, br_end) {
+		reverse_foreach (br, bad) {
 			grammar.rule_by_number.erase(grammar.rule_by_number.begin() + (*br)->number);
 		}
-		foreach (CG3::RuleVector, bad, br, br_end) {
-			(*br)->number = grammar.rule_by_number.size();
-			grammar.rule_by_number.push_back(*br);
+		for (auto br : bad) {
+			br->number = grammar.rule_by_number.size();
+			grammar.rule_by_number.push_back(br);
 		}
 		std::cerr << "Optimizer moved " << bad.size() << " rules." << std::endl;
 		grammar.reindex();
@@ -393,7 +386,7 @@ int main(int argc, char* argv[]) {
 			writer.writeGrammar(gout);
 
 			if (options[VERBOSE].doesOccur) {
-				std::cerr << "Writing textual grammar took " << (clock()-main_timer)/(double)CLOCKS_PER_SEC << " seconds." << std::endl;
+				std::cerr << "Writing textual grammar took " << (clock() - main_timer) / (double)CLOCKS_PER_SEC << " seconds." << std::endl;
 			}
 			main_timer = clock();
 		}
@@ -409,7 +402,7 @@ int main(int argc, char* argv[]) {
 			writer.writeBinaryGrammar(gout);
 
 			if (options[VERBOSE].doesOccur) {
-				std::cerr << "Writing binary grammar took " << (clock()-main_timer)/(double)CLOCKS_PER_SEC << " seconds." << std::endl;
+				std::cerr << "Writing binary grammar took " << (clock() - main_timer) / (double)CLOCKS_PER_SEC << " seconds." << std::endl;
 			}
 			main_timer = clock();
 		}
@@ -425,7 +418,7 @@ int main(int argc, char* argv[]) {
 	u_cleanup();
 
 	if (options[VERBOSE].doesOccur) {
-		std::cerr << "Cleanup took " << (clock()-main_timer)/(double)CLOCKS_PER_SEC << " seconds." << std::endl;
+		std::cerr << "Cleanup took " << (clock() - main_timer) / (double)CLOCKS_PER_SEC << " seconds." << std::endl;
 	}
 
 	return status;
@@ -472,6 +465,9 @@ void GAppSetOpts(CG3::GrammarApplicator& applicator, UConverter *conv) {
 	}
 	if (options[TRACE].doesOccur) {
 		applicator.trace = true;
+		if (options[TRACE].value) {
+			CG3::GAppSetOpts_ranged(options[TRACE].value, applicator.trace_rules, false);
+		}
 	}
 	if (options[TRACE_NAME_ONLY].doesOccur) {
 		applicator.trace = true;
@@ -507,13 +503,12 @@ void GAppSetOpts(CG3::GrammarApplicator& applicator, UConverter *conv) {
 		else {
 			UErrorCode status = U_ZERO_ERROR;
 			size_t sn = strlen(options[RULE].value);
-			UChar *buf = new UChar[sn*3];
+			UChar *buf = new UChar[sn * 3];
 			buf[0] = 0;
-			ucnv_toUChars(conv, buf, sn*3, options[RULE].value, sn, &status);
+			ucnv_toUChars(conv, buf, sn * 3, options[RULE].value, sn, &status);
 
-			const_foreach (CG3::RuleVector, applicator.grammar->rule_by_number, riter, riter_end) {
-				const CG3::Rule *rule = *riter;
-				if (rule->name && u_strcmp(rule->name, buf) == 0) {
+			for (auto rule : applicator.grammar->rule_by_number) {
+				if (rule->name == buf) {
 					applicator.valid_rules.push_back(rule->number);
 				}
 			}
