@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2015, GrammarSoft ApS
+* Copyright (C) 2007-2017, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -26,23 +26,23 @@
 #include <vector>
 #include <algorithm>
 #include <functional>
-#include <stdint.h> // C99 or C++0x or C++ TR1 will have this header. ToDo: Change to <cstdint> when C++0x broader support gets under way.
+#include <cstdint>
 
 namespace CG3 {
-	namespace detail {
-		template<typename ForwardIt, typename Comp>
-		bool is_sorted(ForwardIt first, ForwardIt last, Comp comp) {
-			if (first != last) {
-				ForwardIt next = first;
-				while (++next != last) {
-					if (comp(*next, *first))
-						return false;
-					first = next;
-				}
+namespace detail {
+	template<typename ForwardIt, typename Comp>
+	bool is_sorted(ForwardIt first, ForwardIt last, Comp comp) {
+		if (first != last) {
+			ForwardIt next = first;
+			while (++next != last) {
+				if (comp(*next, *first))
+					return false;
+				first = next;
 			}
-			return true;
 		}
+		return true;
 	}
+}
 
 template<typename T, typename Comp = std::less<T> >
 class sorted_vector {
@@ -65,7 +65,7 @@ public:
 	}
 	#endif
 
-	std::pair<iterator,bool> insert(T t) {
+	std::pair<iterator, bool> insert(T t) {
 		if (elements.empty()) {
 			elements.push_back(t);
 			return std::make_pair(elements.begin(), true);
@@ -84,7 +84,7 @@ public:
 		size_t at = std::distance(elements.begin(), it);
 		if (it == elements.end() || comp(*it, t) || comp(t, *it)) {
 			elements.insert(it, t);
-			return std::make_pair(elements.begin()+at, true);
+			return std::make_pair(elements.begin() + at, true);
 		}
 		return std::make_pair(elements.begin() + at, false);
 	}
@@ -116,8 +116,8 @@ public:
 		elements.erase(it, elements.end());
 	}
 
-	bool push_back(T t) {
-		return insert(t);
+	void push_back(T t) {
+		insert(t);
 	}
 
 	bool erase(T t) {
@@ -143,6 +143,13 @@ public:
 		return elements.erase(elements.begin() + o);
 	}
 
+	template<typename It>
+	void erase(It b, It e) {
+		for (; b != e; ++b) {
+			erase(*b);
+		}
+	}
+
 	const_iterator find(T t) const {
 		if (elements.empty()) {
 			return elements.end();
@@ -238,6 +245,14 @@ public:
 		elements.clear();
 	}
 
+	void sort() {
+		std::sort(elements.begin(), elements.end(), Comp());
+	}
+
+	void pop_back() {
+		elements.pop_back();
+	}
+
 	container& get() {
 		return elements;
 	}
@@ -248,7 +263,6 @@ private:
 };
 
 typedef sorted_vector<uint32_t> uint32SortedVector;
-
 }
 
 #endif
