@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2015, GrammarSoft ApS
+* Copyright (C) 2007-2017, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -24,16 +24,12 @@
 #define c6d28b7452ec699b_STDAFX_H
 
 #ifdef _MSC_VER
-	// warning C4258: definition from the for loop is ignored; the definition from the enclosing scope is used
-	#pragma warning (disable: 4258)
-	#pragma conform(forScope, on)
-	// warning C4428: universal-character-name encountered in source
-	#pragma warning (disable: 4428)
 	// warning C4512: assignment operator could not be generated
 	#pragma warning (disable: 4512)
-	// warning C4480: nonstandard extension used: specifying underlying type for enum
-	// 'cause that is actually standard in C++11
-	#pragma warning (disable: 4480)
+	// warning C4456: declaration hides previous local declaration
+	#pragma warning (disable: 4456)
+	// warning C4458: declaration hides class member
+	#pragma warning (disable: 4458)
 #endif
 
 #include <exception>
@@ -53,6 +49,8 @@
 #include <string>
 #include <set>
 #include <map>
+#include <unordered_map>
+#include <unordered_set>
 #include <stack>
 #include <limits>
 #include <ctime>
@@ -61,7 +59,7 @@
 #include <cassert>
 #include <ciso646>
 #include <sys/stat.h>
-#include <stdint.h> // C99 or C++0x or C++ TR1 will have this header. ToDo: Change to <cstdint> when C++0x broader support gets under way.
+#include <cstdint>
 #include <cycle.h>
 
 // cycle.h doesn't know all platforms (such as ARM), so fall back on clock()
@@ -77,18 +75,17 @@
 	#define HAVE_TICK_COUNTER
 #endif
 
-#include <boost/unordered_set.hpp>
-#include <boost/unordered_map.hpp>
 #include <boost/container/flat_set.hpp>
 #include <boost/container/flat_map.hpp>
 #include <boost/dynamic_bitset.hpp>
-#include <boost/scoped_ptr.hpp>
-#include <boost/typeof/typeof.hpp>
-#include <boost/foreach.hpp>
-#define boost_foreach BOOST_FOREACH
-#define boost_reverse_foreach BOOST_REVERSE_FOREACH
-#define stdext boost
-#define hash_map unordered_map
+
+#define foreach(iter, container) \
+	if (!(container).empty())    \
+		for (auto iter = (container).begin(), iter##_end = (container).end(); iter != iter##_end; ++iter)
+
+#define reverse_foreach(iter, container) \
+	if (!(container).empty())            \
+		for (auto iter = (container).rbegin(), iter##_end = (container).rend(); iter != iter##_end; ++iter)
 
 #ifdef _WIN32
 	#include <winsock.h> // for hton() and family.
@@ -113,13 +110,12 @@
 #include <unicode/ubrk.h>
 
 namespace CG3 {
-	typedef std::basic_string<UChar> UString;
-	typedef std::vector<UString> UStringVector;
-	typedef std::vector<uint32_t> uint32Vector;
-	namespace bc = ::boost::container;
+typedef std::basic_string<UChar> UString;
+typedef std::vector<UString> UStringVector;
+typedef std::vector<uint32_t> uint32Vector;
+namespace bc = ::boost::container;
 }
 
-#include "macros.hpp"
 #include "inlines.hpp"
 #include "uextras.hpp"
 #include "flat_unordered_map.hpp"
